from __future__ import division
import numpy as np
import scipy.special as ss

def degtorad(d):
  """
    Convert degrees into radians.
    
    Parameters
    ----------
    d : float or array
        Angle in degrees.
    
    Returns
    -------
    Angle : float or array
        The angle converted into radians.
  """
  return (d/180.0)*np.pi

def radtodeg(r):
  """
    Convert radians into degrees.
    
    Parameters
    ----------
    d : float or array
        Angle in radians.
    
    Returns
    -------
    Angle : float or array
        The angle converted into degrees.
  """
  return (r/np.pi)*180.


def farat(x, y):
    """
    Compute ratio of factorials.
    
    Computes x!/y! via ln(x!) - ln(y!) to avoid
    numerical overflow.
    
    Parameters
    ----------
    x : int, float
        The factorial of x is the numerator
    y : int, float
        The factorial of y is the denumerator
    
    Returns
    -------
    Ratio : float
        The ratio x!/y! (not the logarithm).
    """
    lnr = ss.gammaln(x+1) - ss.gammaln(y+1)
    return np.exp(lnr)




from __future__ import print_function, division
from PyAstronomy import funcFit as fuf
import numpy as np


class AtanProfile(fuf.OneDFit):
    r"""
    A profile based on the arc tangent function.

    This class implements the following profile:

    .. math::
      f(x) = \frac{A}{2\arctan(\sigma)} \times \left(\arctan\left(\frac{x-\mu}{scale} + \sigma\right) +
             \arctan\left(-\frac{x-\mu}{scale} + \sigma\right)\right) +
             \mu \times x + off

    which can provide a relatively flat top and steep edges.

    *Fit parameters*
      -  `A`    - The amplitude. In this case, the height (not the area under)
                  the profile reached for :math:`x=0`. Note that for
                  :math:`\mu \not = 0` the highest point may be elsewhere,
                  which is neglected here.
      - `scale` - A scale parameter affecting the width of the profile. Note,
                  however, that also :math:`\sigma` affects the width.
      - `mu`    - The center of the profile.
      - `off`   - An offset
      - `lin`   - A gradient in the offset.

    The width of the profile may be approximated by the inflection points, which
    are given by

    .. math::
      \frac{\partial^2 f(x)}{\partial x^2} = 0 \rightarrow
      x_{1,2} = \mu \pm\frac{scale}{3}\left(-3+3\sigma^2+6\sqrt{\sigma^4+\sigma^2+1}\right)^{1/2}

    """

    def __init__(self):
        fuf.OneDFit.__init__(self, ["scale", "sig", "mu", "A", "off", "lin"])

    def evaluate(self, x):
        r"""
        Calculates and returns model according to the
        current parameter values.

        Parameters
        ----------
        x : Array
            The positions at which to evaluate the model.
        """
        # Shift by mu
        x = x - self["mu"]
        # The heart of the profile
        y = np.arctan(x/self["scale"] + self["sig"]) + \
            np.arctan(-x/self["scale"] + self["sig"])
        # Make the highest point (actually most extreme point)
        # equal to A
        y *= (self["A"] / (2.*np.arctan(self["sig"])))

        # Add offset and gradient
        y += self["off"]
        y += self["lin"] * (x + self["mu"])
        return y

    def inflectionPoints(self):
        r"""
        Calculate the inflection points.

        The inflection points of the profile depend on
        both :math:`\sigma` and :math:`\mu`.

        Returns
        -------
        Inflection points : tuple
            Locations of the inflection points. Smaller one first.
        """
        d = abs(self["scale"])/3.0 * \
            np.sqrt(-3. + 3.*self["sig"]**2 + 6. *
                    np.sqrt(self["sig"]**4 + self["sig"]**2 + 1.0))
        return self["mu"]-d, self["mu"]+d


class AtanProfileDamped(fuf.OneDFit):
    r"""
    A profile based on the arc tangent function.

    This class implements the following profile:

    .. math::
      d(x) = f(x) \times H(|x-\mu| - |ifp-\mu|) \times
             \exp\left(\frac{|x-\mu| - |ifp-\mu|}{\tau}\right) +
             \mu \times x + off

    Here :math:`f(x)` is the profile described in :py:class:`AtanProfile`,
    H denotes the Heaviside function, and ifp is the location of the
    inflection point. The parameter :math:`\tau` can be used to provide
    an additional drop at the edges of the profile.  


    *Fit parameters*
      -  `A`    - The amplitude. In this case, the height (not the area under)
                  the profile reached for :math:`x=0`. Note that for
                  :math:`\mu \not = 0` the highest point may be elsewhere,
                  which is neglected here.
      - `scale` - A scale parameter affecting the width of the profile. Note,
                  however, that also :math:`\sigma` affects the width.
      - `tau`   - This parameter controls an additional drop at the edges
                  of the profile.
      - `mu`    - The center of the profile.
      - `off`   - An offset
      - `lin`   - A gradient in the offset.

    The width of the profile may be approximated by the inflection points, which
    are given by

    .. math::
      \frac{\partial^2 f(x)}{\partial x^2} = 0 \rightarrow
      x_{1,2} = \mu \pm\frac{scale}{3}\left(-3+3\sigma^2+6\sqrt{\sigma^4+\sigma^2+1}\right)^{1/2}

    """

    def __init__(self):
        fuf.OneDFit.__init__(
            self, ["scale", "sig", "mu", "A", "off", "lin", "tau"])

    def evaluate(self, x):
        r"""
        Calculates and returns model according to the
        current parameter values.

        Parameters
        ----------
        x : Array
            The positions at which to evaluate the model.
        """
        # Shift by mu
        x = x - self["mu"]
        # The heart of the profile
        y = np.arctan(x/self["scale"] + self["sig"]) + \
            np.arctan(-x/self["scale"] + self["sig"])
        # Make the highest point (actually most extreme point)
        # equal to A
        y *= (self["A"] / (2.*np.arctan(self["sig"])))

        # Produce additional drop
        difp = abs(self.inflectionPoints()[0] - self["mu"])
        indi = np.where(np.abs(x) > difp)[0]
        y[indi] *= np.exp(-np.abs(np.abs(x[indi])-difp)**2/self["tau"])

        # Add offset and gradient
        y += self["off"]
        y += self["lin"] * (x + self["mu"])
        return y

    def inflectionPoints(self):
        r"""
        Calculate the inflection points.

        The inflection points of the profile depend on
        both :math:`\sigma` and :math:`\mu`.

        Returns
        -------
        Inflection points : tuple
            Locations of the inflection points. Smaller one first.
        """
        d = abs(self["scale"])/3.0 * \
            np.sqrt(-3. + 3.*self["sig"]**2 + 6. *
                    np.sqrt(self["sig"]**4 + self["sig"]**2 + 1.0))
        return self["mu"]-d, self["mu"]+d



import numpy as np

def parallacticAngle(t, gl, dec):
    """
    Calculate the parallactic angle
    
    The parallactic angle is the position angle of the zenith
    at the target.
    
    Notes
    -----
    Useful links
    
    `Parallactic Angle Observations (Keck) <https://www2.keck.hawaii.edu/inst/common/parallactic.html>`_
    
    `What Wiki has to say <https://en.wikipedia.org/wiki/Parallactic_angle>`_
    
    
    Parameters
    ----------
    t : float or array
        Hour angle [deg]
    gl : float
        Latitude of the observer [deg]
    dec : float
        Declination of the target [deg]
    
    Returns
    -------
    q : float or array
        The parallactic angle [deg]
        
    """
    t = np.deg2rad(t)
    gl = np.deg2rad(gl)
    dec = np.deg2rad(dec)
    
    up = np.sin(t)
    down = np.tan(gl)*np.cos(dec) - np.sin(dec)*np.cos(t)
    r = np.arctan2(up,down)
    return np.rad2deg(r)




import unittest
class TestSanityOfNotebook(unittest.TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass
        
    def testsanity_example(self):
        #!/usr/bin/env python
        # coding: utf-8
        
        # Custom linear model
        # ==================
        
        # In[1]:
        
        
        import numpy as np
        import matplotlib.pylab as plt
        from PyAstronomy import funcFit2 as fuf2
        import scipy.optimize as sco
        import matplotlib
        matplotlib.use('Agg')
        
        np.random.seed(1234)
        
        class LinMod(fuf2.MBOEv):
            """ Linear model """
        
            def __init__(self):
                # 'pars' specifies parameter names in the model
                fuf2.MBOEv.__init__(self, pars=["const", "slope"], rootName="LinMod")
        
            def evaluate(self, x, *args):
                """
                Evaluate model at 'x' here and return y = const + x*slope
                
                args receives the remianing arguments specified in the call to fmin but
                not needed here
                """
                return self["const"] + x * self["slope"]
        
        
        # Instantiate model
        lm = LinMod()
        
        lm["slope"] = 1.1
        lm["const"] = -0.5
        
        # Get some 'data' and add Gaussian noise with STD 10
        x = np.arange(15.)
        y = lm.evaluate(x) + np.random.normal(0,1,len(x))
        yerr = np.ones_like(x)
        
        print("Input values for mock data: ", lm.parameters())
        print()
        
        lm.thaw(["slope", "const"])
        
        # Because ln inherited from MBOEv, it has a default chi-square objective function
        fr = sco.fmin(lm.chisqr, x0=lm.freeParamVals(), args=(x,y,yerr))
        lm.setFreeParamVals(fr)
        
        lm.parameterSummary()
        
        plt.errorbar(x, y, yerr=yerr, fmt='b+')
        plt.plot(x, lm.evaluate(x), 'r--')
        plt.show()
        



import numpy as np
from PyAstronomy.pyaC import pyaErrors as PE
from PyAstronomy.pyasl import _ic


def crosscorrRV(
    w,
    f,
    tw,
    tf,
    rvmin,
    rvmax,
    drv,
    mode="doppler",
    skipedge=0,
    edgeTapering=None,
    weights=None,
    meanwvl=None,
):
    r"""
    Cross-correlate a spectrum with a template.

    The algorithm implemented here works as follows: For
    each RV shift to be considered, the wavelength axis
    of the template is shifted, either linearly or using
    a proper Doppler shift depending on the `mode`. The
    shifted template is then linearly interpolated at
    the wavelength points of the observation
    (spectrum) to calculate the cross-correlation function.

    If N data points :math:`f_i` at wavelengths :math:`w_i` are given, the cross
    correlation function :math:`CC(v_j)`, depending on the velocity :math:`v_j` and
    optional weights :math:`\alpha_i` is calculated as:

    .. math::

        CC(v_j) = \sum_{i=1}^N \alpha_i \times \left(f_i \times t(w_i-\Delta_{i,j}) \right)

    If the mode is `lin`, the shift is implemented as :math:`\Delta_{i,j} = \bar{w} (v_j/c)`, where
    :math:`\bar{w}` is the mean wavelength (i.e., the mean of the input w unless meanwvl is specified).
    For any velocity, the shift is a constant across the entire wavelength
    axis. If the mode is `doppler`, the shift is implemented as
    :math:`\Delta_{i,j} = w_i (v_j/c)`, which depends both on the wavelength and the velocity.
    The evaluation of the shifted template :math:`t(w_i-\Delta_{i,j})` is carried out using linear interpolation.

    If no weights are specified, :math:`\alpha_i = 1` is adopted. If uncertainties, :math:`\sigma_i`, for the
    :math:`f_i` are known, a useful choice for the weights is :math:`\alpha_i = \sigma_i^{-2}`, because this makes
    maximizing the CCF equivalent to maximizing the likelihood 
    (see, e.g., `Lookwood et al. 2014, ApJ 783 <https://ui.adsabs.harvard.edu/abs/2014ApJ...783L..29L/abstract>`_).

    Parameters
    ----------
    w : array
        The wavelength axis of the observation.
    f : array
        The flux axis of the observation.
    tw : array
        The wavelength axis of the template.
    tf : array
        The flux axis of the template.
    rvmin : float
        Minimum radial velocity for which to calculate
        the cross-correlation function [km/s].
    rvmax : float
        Maximum radial velocity for which to calculate
        the cross-correlation function [km/s].
    drv : float
        The width of the radial-velocity steps to be applied
        in the calculation of the cross-correlation
        function [km/s].
    mode : string, {lin, doppler}, optional
        The mode determines how the wavelength axis will be
        modified to represent a RV shift. If "lin" is specified,
        a mean wavelength shift will be calculated based on the
        mean wavelength of the observation. The wavelength axis
        will then be shifted by that amount. If "doppler" is
        specified (the default), the wavelength axis will
        properly be Doppler-shifted.
    skipedge : int, optional
        If larger zero, the specified number of bins will be
        skipped from the begin and end of the observation. This
        may be useful if the template does not provide sufficient
        coverage of the observation.
    edgeTapering : float or tuple of two floats
        If not None, the method will "taper off" the edges of the
        observed spectrum by multiplying with a sine function. If a float number
        is specified, this will define the width (in wavelength units)
        to be used for tapering on both sides. If different tapering
        widths shall be used, a tuple with two (positive) numbers
        must be given, specifying the width to be used on the low- and
        high wavelength end. If a nonzero 'skipedge' is given, it
        will be applied first. Edge tapering can help to avoid
        edge effects (see, e.g., Gullberg and Lindegren 2002, A&A 390).
    weights : array of floats, optional
        If given, it specifies weights for individual data points (given by f).
        A possible choice for weights is 1/sigma_i**2 where sigma_i is the
        uncertainty of the i-th data point.
    meanwvl : float, optional
        If given, this value will be adopted for the mean wavelength
        :math:`\bar{w}` to calculate shifts in 'lin' mode instead of
        the mean of the input array w.

    Returns
    -------
    dRV : array
        The RV axis of the cross-correlation function. The radial
        velocity refer to a shift of the template, i.e., positive
        values indicate that the template has been red-shifted and
        negative numbers indicate a blue-shift of the template.
        The numbers are given in km/s.
    CC : array
        The cross-correlation function.
    """
    if not _ic.check["scipy"]:
        raise (
            PE.PyARequiredImport(
                "This routine needs scipy (.interpolate.interp1d).",
                where="crosscorrRV",
                solution="Install scipy",
            )
        )
    import scipy.interpolate as sci

    if weights is None:
        weights = np.ones_like(w)
    else:
        weights = weights.copy()

    if (w.shape != f.shape) or (w.shape != weights.shape):
        raise (
            PE.PyAValError(
                f"w, f, and weights must have the same shape. Given shapes are (w,f,weights): {w.shape}, {f.shape}, {weights.shape}",
                where="crosscorrRV",
                solution="Adjust arrays (w,f) and/or weights if specified.",
            )
        )

    # Copy and cut wavelength and flux arrays
    w, f = w.copy(), f.copy()
    if skipedge > 0:
        w, f = w[skipedge:-skipedge], f[skipedge:-skipedge]
        weights = weights[skipedge:-skipedge]

    if edgeTapering is not None:
        # Smooth the edges using a sine
        if isinstance(edgeTapering, (float, int)):
            edgeTapering = [float(edgeTapering), float(edgeTapering)]
        if len(edgeTapering) != 2:
            raise (
                PE.PyAValError(
                    "'edgeTapering' must be a float or a list of two floats.",
                    where="crosscorrRV",
                )
            )
        if edgeTapering[0] < 0.0 or edgeTapering[1] < 0.0:
            raise (
                PE.PyAValError(
                    "'edgeTapering' must be (a) number(s) >= 0.0.", where="crosscorrRV"
                )
            )
        # Carry out edge tapering (left edge)
        indi = np.where(w < w[0] + edgeTapering[0])[0]
        f[indi] *= np.sin((w[indi] - w[0]) / edgeTapering[0] * np.pi / 2.0)
        # Carry out edge tapering (right edge)
        indi = np.where(w > (w[-1] - edgeTapering[1]))[0]
        f[indi] *= np.sin(
            (w[indi] - w[indi[0]]) / edgeTapering[1] * np.pi / 2.0 + np.pi / 2.0
        )

    # Speed of light in km/s
    c = 299792.458
    # Check order of rvmin and rvmax
    if rvmax <= rvmin:
        raise (
            PE.PyAValError(
                "rvmin needs to be smaller than rvmax.",
                where="crosscorrRV",
                solution="Change the order of the parameters.",
            )
        )
    # Check whether template is large enough
    if mode == "lin":
        if meanwvl is None:
            meanWl = np.mean(w)
        else:
            meanWl = meanwvl
        dwlmax = meanWl * (rvmax / c)
        dwlmin = meanWl * (rvmin / c)
        if (tw[0] + dwlmax) > w[0]:
            raise (
                PE.PyAValError(
                    "The minimum wavelength is not covered by the template for all indicated RV shifts.",
                    where="crosscorrRV",
                    solution=["Provide a larger template",
                              "Try to use skipedge"],
                )
            )
        if (tw[-1] + dwlmin) < w[-1]:
            raise (
                PE.PyAValError(
                    "The maximum wavelength is not covered by the template for all indicated RV shifts.",
                    where="crosscorrRV",
                    solution=["Provide a larger template",
                              "Try to use skipedge"],
                )
            )
    elif mode == "doppler":
        # Ensure that the template covers the entire observation for all shifts
        maxwl = tw[-1] * (1.0 + rvmin / c)
        minwl = tw[0] * (1.0 + rvmax / c)
        if minwl > w[0]:
            raise (
                PE.PyAValError(
                    "The minimum wavelength is not covered by the template for all indicated RV shifts.",
                    where="crosscorrRV",
                    solution=["Provide a larger template",
                              "Try to use skipedge"],
                )
            )
        if maxwl < w[-1]:
            raise (
                PE.PyAValError(
                    "The maximum wavelength is not covered by the template for all indicated RV shifts.",
                    where="crosscorrRV",
                    solution=["Provide a larger template",
                              "Try to use skipedge"],
                )
            )
    else:
        raise (
            PE.PyAValError(
                "Unknown mode: " + str(mode),
                where="crosscorrRV",
                solution="See documentation for available modes.",
            )
        )
    # Calculate the cross correlation
    drvs = np.arange(rvmin, rvmax, drv)
    cc = np.zeros(len(drvs))
    for i, rv in enumerate(drvs):
        if mode == "lin":
            # Shift the template linearly
            fi = sci.interp1d(tw + meanWl * (rv / c), tf)
        elif mode == "doppler":
            # Apply the Doppler shift
            fi = sci.interp1d(tw * (1.0 + rv / c), tf)
        # Shifted template evaluated at location of spectrum
        cc[i] = np.sum(f * fi(w) * weights)
    return drvs, cc



from __future__ import division
import unittest
import numpy as np


class TestpalMandelSanity(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def _rr(self, low, high):
        return np.random.random() * (high - low) + low

    def testsanity_comapare(self):
        """
        Compare Mandel's occultquad and the Pal results.
        """
        from PyAstronomy.modelSuite import palTrans
        from PyAstronomy.modelSuite import forTrans

        pal = palTrans.PalLC()
        onl = forTrans.MandelAgolLC()
        x = np.linspace(0, 3, 500)
        for counter in range(5):
            linLimb = self._rr(0, 1)
            par = {
                "p": self._rr(0.0, 0.2),
                "a": self._rr(2.0, 5.0),
                "i": self._rr(86, 94),
                "linLimb": linLimb,
                "quadLimb": self._rr(0, 1.0 - linLimb),
                "T0": self._rr(0, 1),
                "per": self._rr(2, 3),
                "b": self._rr(0, 0.1),
            }
            pal.assignValue(par)
            onl.assignValue(par)
            yp = pal.evaluate(x)
            ym = onl.evaluate(x)

            np.testing.assert_almost_equal(
                yp,
                ym,
                5,
                "The Pal and Mandel models differ."
                + "\n\n"
                + "\n".join(pal.parameterSummary(False))
                + "\n\n"
                + "\n".join(onl.parameterSummary(False)),
                True,
            )

    def testsanity_basicCirc(self):
        """
        Checking basic MA and Pal transit configuration. Must be identical.
        """
        from PyAstronomy.modelSuite import XTran as xt
        import numpy as np

        mac = xt.forTrans.MandelAgolLC()
        mak = xt.forTrans.MandelAgolLC(orbit="keplerian")

        mac["p"] = 0.16
        mac["per"] = 2.1
        mac["linLimb"] = 0.37
        mac["a"] = 5.4
        mac["i"] = 90

        for p in ["p", "per", "linLimb", "a", "i"]:
            mak[p] = mac[p]

        t = np.linspace(0, 4, 1001)
        cc = mac.evaluate(t)
        ck = mak.evaluate(t)
        self.assertAlmostEqual(
            np.max(np.abs(cc - ck)),
            0.0,
            delta=1e-8,
            msg="MandelAgol: Default for circ and kep orbit not identical",
        )

        pa = xt.palTrans.PalLC()
        for p in ["p", "per", "linLimb", "a", "i"]:
            pa[p] = mac[p]
        cp = pa.evaluate(t)
        self.assertAlmostEqual(
            np.max(np.abs(cc - cp)),
            0.0,
            delta=1e-8,
            msg="MandelAgol vs. Pal: Default for circ orbit not identical",
        )

        t = np.array([0, 1.05, 2.099])
        cp = pa.evaluate(t)
        np.testing.assert_array_equal(
            np.array([0, 2]),
            pa._intrans,
            err_msg="_zlistCirc: Problem with _intrans",
            verbose=True,
        )
        np.testing.assert_array_equal(
            np.array([1]),
            pa._inocc,
            err_msg="_zlistCirc: Problem with _inocc",
            verbose=True,
        )



# -*- coding: utf-8 -*-
from __future__ import print_function, division
import numpy as np
from .idlCirrange import cirrange
from .eq2hor import nutate
import six.moves as smo


def moonpos(jd, radian=False):
  """
    Computes RA and DEC of the Moon at given Julian date(s).
    
    Parameters
    ----------
    jd : float or array
        The Julian date.
    radian : boolean, optional
        If True, results are returned in RADIAN instead
        of DEGREES. Default is False.
    
    Returns
    -------
    RA : float or array
        Right ascension of the moon for given JD(s) in DEGREES.
    DEC : float or array
        Declination of the moon for given JD(s) in DEGREES.
    DISTANCE : float or array
        Distance of the moon from the Earth for given JD(s) in KILOMETERS.
    GEOLONGITUDE : float or array
        Apparent longitude of the moon for given JD(s) in DEGREES.
    GEOLATITUDE : float or array
        Apparent latitude of the moon for given JD(s) in DEGREES.

    Notes
    -----
    
    .. note:: This function was ported from the IDL Astronomy User's Library.

    :IDL - Documentation:  

    PRO MOONPOS, jd, ra, dec, dis, geolong, geolat, RADIAN = radian
    
    NAME:                                     
          MOONPOS
    PURPOSE:
          To compute the RA and Dec of the Moon at specified Julian date(s).
    
    CALLING SEQUENCE:
          MOONPOS, jd, ra, dec, dis, geolong, geolat, [/RADIAN ]
    
    INPUTS:
          JD - Julian ephemeris date, scalar or vector, double precision suggested
    
    OUTPUTS:
          Ra  - Apparent right ascension of the moon in DEGREES, referred to the
                  true equator of the specified date(s) 
          Dec - The declination of the moon in DEGREES 
          Dis - The Earth-moon distance in kilometers (between the center of the
                Earth and the center of the Moon).
          Geolong - Apparent longitude of the moon in DEGREES, referred to the
                  ecliptic of the specified date(s)
          Geolat - Apparent longitude of the moon in DEGREES, referred to the
                  ecliptic of the specified date(s)
    
          The output variables will all have the same number of elements as the
          input Julian date vector, JD.   If JD is a scalar then the output 
          variables will be also.
    
    OPTIONAL INPUT KEYWORD:
          /RADIAN - If this keyword is set and non-zero, then all output variables 
                  are given in Radians rather than Degrees
    
    EXAMPLES:
          (1) Find the position of the moon on April 12, 1992
    
          IDL> jdcnv,1992,4,12,0,jd    ;Get Julian date
          IDL> moonpos, jd, ra ,dec     ;Get RA and Dec of moon
          IDL> print,adstring(ra,dec,1)
                  ==> 08 58 45.23  +13 46  6.1
    
          This is within 1" from the position given in the Astronomical Almanac
          
          (2) Plot the Earth-moon distance for every day at 0 TD in July, 1996
    
          IDL> jdcnv,1996,7,1,0,jd                   ;Get Julian date of July 1
          IDL> moonpos,jd+dindgen(31), ra, dec, dis  ;Position at all 31 days
          IDL> plot,indgen(31),dis, /YNOZ
    
    METHOD:
          Derived from the Chapront ELP2000/82 Lunar Theory (Chapront-Touze' and
          Chapront, 1983, 124, 50), as described by Jean Meeus in Chapter 47 of
          ``Astronomical Algorithms'' (Willmann-Bell, Richmond), 2nd edition, 
          1998.    Meeus quotes an approximate accuracy of 10" in longitude and
          4" in latitude, but he does not give the time range for this accuracy.
    
          Comparison of this IDL procedure with the example in ``Astronomical
          Algorithms'' reveals a very small discrepancy (~1 km) in the distance 
          computation, but no difference in the position calculation.
    
          This procedure underwent a major rewrite in June 1996, and the new
          calling sequence is *incompatible with the old* (e.g. angles now 
          returned in degrees instead of radians).
    
    PROCEDURES CALLED:
          CIRRANGE, ISARRAY(), NUTATE, TEN()  - from IDL Astronomy Library
          POLY() - from IDL User's Library
    MODIFICATION HISTORY:
          Written by Michael R. Greason, STX, 31 October 1988.
          Major rewrite, new (incompatible) calling sequence, much improved 
                  accuracy,       W. Landsman   Hughes STX      June 1996
          Added /RADIAN keyword  W. Landsman August 1997
          Converted to IDL V5.0   W. Landsman   September 1997
          Use improved expressions for L',D,M,M', and F given in 2nd edition of
               Meeus (very slight change),  W. Landsman    November 2000
          Avoid 32767 overflow   W. Landsman January 2005
  """

  jd = np.array(jd, ndmin=1)
  time = (jd - 2451545.0)/36525.0
  
  d_lng = np.array([0,2,2,0,0,0,2,2,2,2,0,1,0,2,0,0,4,0,4,2,2,1,1,2,2,4,2,0,2,2,1,2,0,0, \
           2,2,2,4,0,3,2,4,0,2,2,2,4,0,4,1,2,0,1,3,4,2,0,1,2,2])

  m_lng = np.array([0,0,0,0,1,0,0,-1,0,-1,1,0,1,0,0,0,0,0,0,1,1,0,1,-1,0,0,0,1,0,-1,0, \
           -2,1,2,-2,0,0,-1,0,0,1,-1,2,2,1,-1,0,0,-1,0,1,0,1,0,0,-1,2,1,0,0])

  mp_lng = np.array([1,-1,0,2,0,0,-2,-1,1,0,-1,0,1,0,1,1,-1,3,-2,-1,0,-1,0,1,2,0,-3,-2, \
            -1,-2,1,0,2,0,-1,1,0,-1,2,-1,1,-2,-1,-1,-2,0,1,4,0,-2,0,2,1,-2,-3,2,1,-1,3,-1])

  f_lng = np.array([0,0,0,0,0,2,0,0,0,0,0,0,0,-2,2,-2,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0, \
           0,0,0,-2,2,0,2,0,0,0,0,0,0,-2,0,0,0,0,-2,-2,0,0,0,0,0,0,0,-2])

  sin_lng = np.array([6288774,1274027,658314,213618,-185116,-114332,58793,57066,53322, \
             45758,-40923,-34720,-30383,15327,-12528,10980,10675,10034,8548,-7888,-6766, \
             -5163,4987,4036,3994,3861,3665,-2689,-2602,2390,-2348,2236,-2120,-2069,2048, \
             -1773,-1595,1215,-1110,-892,-810,759,-713,-700,691,596,549,537,520,-487, \
             -399,-381,351,-340,330,327,-323,299,294,0.0])

  cos_lng = np.array([-20905355,-3699111,-2955968,-569925,48888,-3149,246158,-152138, \
             -170733,-204586,-129620,108743,104755,10321,0,79661,-34782,-23210,-21636, \
             24208,30824,-8379,-16675,-12831,-10445,-11650,14403,-7003,0,10056,6322, \
             -9884,5751,0,-4950,4130,0,-3958,0,3258,2616,-1897,-2117,2354,0,0,-1423, \
             -1117,-1571,-1739,0,-4421,0,0,0,0,1165,0,0,8752.0])

  d_lat = np.array([0,0,0,2,2,2,2,0,2,0,2,2,2,2,2,2,2,0,4,0,0,0,1,0,0,0,1,0,4,4,0,4,2,2, \
                    2,2,0,2,2,2,2,4,2,2,0,2,1,1,0,2,1,2,0,4,4,1,4,1,4,2])

  m_lat = np.array([0,0,0,0,0,0,0,0,0,0,-1,0,0,1,-1,-1,-1,1,0,1,0,1,0,1,1,1,0,0,0,0,0,0, \
                    0,0,-1,0,0,0,0,1,1,0,-1,-2,0,1,1,1,1,1,0,-1,1,0,-1,0,0,0,-1,-2])

  mp_lat = np.array([0,1,1,0,-1,-1,0,2,1,2,0,-2,1,0,-1,0,-1,-1,-1,0,0,-1,0,1,1,0,0,3,0, \
                    -1,1, -2,0,2,1,-2,3,2,-3,-1,0,0,1,0,1,1,0,0,-2,-1,1,-2,2,-2,-1,1,1,-1,0,0])

  f_lat = np.array([ 1,1,-1,-1,1,-1,1,1,-1,-1,-1,-1,1,-1,1,1,-1,-1,-1,1,3,1,1,1,-1,-1,-1, \
                     1,-1,1,-3,1,-3,-1,-1,1,-1,1,-1,1,1,1,1,-1,3,-1,-1,1,-1,-1,1,-1,1,-1,-1, \
                    -1,-1,-1,-1,1])

  sin_lat = np.array([5128122,280602,277693,173237,55413,46271,32573,17198,9266,8822, \
                      8216,4324,4200,-3359,2463,2211,2065,-1870,1828,-1794,-1749,-1565,-1491, \
                      -1475,-1410,-1344,-1335,1107,1021,833,777,671,607,596,491,-451,439,422, \
                      421,-366,-351,331,315,302,-283,-229,223,223,-220,-220,-185,181,-177,176, \
                      166,-164,132,-119,115,107.0])

  # Mean longitude of the moon referred to mean equinox of the date
  coeff0 = [-1.0/6.5194e7, 1.0/538841.0, -0.0015786, 481267.88123421, 218.3164477]
  lprimed = np.polyval(coeff0, time)*np.pi/180.
  lprimed = cirrange(lprimed, radians=True)
  
  # Mean elongation of the Moon
  coeff1 = [-1.0/1.13065e8, 1.0/545868.0, -0.0018819, 445267.1114034, 297.8501921]
  d = np.polyval(coeff1, time)*np.pi/180.
  d = cirrange(d, radians=True)

  # Sun's mean anomaly
  coeff2 = [1.0/2.449e7, -0.0001536, 35999.0502909, 357.5291092]
  M = np.polyval(coeff2, time)*np.pi/180.
  M = cirrange(M, radians=True)
  
  # Moon's mean anomaly
  coeff3 = [-1.0/1.4712e7, 1.0/6.9699e4, 0.0087414, 477198.8675055, 134.9633964]
  Mprime = np.polyval(coeff3, time)*np.pi/180.
  Mprime = cirrange(Mprime, radians=True)

  # Moon's argument of latitude
  coeff4 = [1.0/8.6331e8, -1.0/3.526e7, -0.0036539, 483202.0175233, 93.2720950]
  F = np.polyval(coeff4, time)*np.pi/180.
  F = cirrange(F, radians=True)

  # Eccentricity of Earth's orbit around the Sun
  E = 1 - 0.002516*time - 7.4e-6*time**2
  E2 = E**2
  ecorr1 = np.where(np.abs(m_lng) == 1)[0]
  ecorr2 = np.where(np.abs(m_lat) == 1)[0]
  ecorr3 = np.where(np.abs(m_lng) == 2)[0]
  ecorr4 = np.where(np.abs(m_lat) == 2)[0]

  # Additional arguments
  A1 = (119.75 + 131.849*time) * np.pi/180.
  A2 = (53.09 + 479264.290*time) * np.pi/180.
  A3 = (313.45 + 481266.484*time) * np.pi/180.
  suml_add = 3958.*np.sin(A1) + 1962.*np.sin(lprimed - F) + 318.*np.sin(A2)
  sumb_add =  -2235.*np.sin(lprimed) + 382.*np.sin(A3) + 175.*np.sin(A1-F) + \
              175.*np.sin(A1 + F) + 127.*np.sin(lprimed - Mprime) - 115.*np.sin(lprimed + Mprime)

  # Sum the periodic terms 
  geolong = np.zeros(jd.size)
  geolat = np.zeros(jd.size)
  dis = np.zeros(jd.size)

  for i in smo.range(jd.size):
    sinlng = sin_lng.copy()
    coslng = cos_lng.copy()
    sinlat = sin_lat.copy()

    sinlng[ecorr1] = E[i]*sinlng[ecorr1]
    coslng[ecorr1] = E[i]*coslng[ecorr1]
    sinlat[ecorr2] = E[i]*sinlat[ecorr2]
    sinlng[ecorr3] = E2[i]*sinlng[ecorr3]
    coslng[ecorr3] = E2[i]*coslng[ecorr3]
    sinlat[ecorr4] = E2[i]*sinlat[ecorr4]

    arg = d_lng*d[i] + m_lng*M[i] +mp_lng*Mprime[i] + f_lng*F[i]
    geolong[i] = lprimed[i]/(np.pi/180.) + ( np.sum( sinlng*np.sin(arg) ) + suml_add[i] )/1.0e6

    dis[i] = 385000.56 + np.sum( coslng*np.cos(arg) )/1.0e3

    arg = d_lat*d[i] + m_lat*M[i] +mp_lat*Mprime[i] + f_lat*F[i]
    geolat[i] = ( np.sum( sinlat*np.sin(arg) ) + sumb_add[i] )/1.0e6

  # Find the nutation in longitude
  nut = nutate(jd)
  nlong = nut[0]
  elong = nut[1]
  geolong += nlong
  geolong = geolong*np.pi/180.    
  geolong = cirrange(geolong, radians=True)    
  
  lamb = geolong.copy()
  beta = geolat*np.pi/180.
  geolong = geolong/(np.pi/180.)
  
  # Find mean obliquity and convert lambda,beta to RA, Dec 
  c = np.array([21.448,-4680.93,-1.55,1999.25,-51.38,-249.67,-39.05,7.12,27.87,5.79,2.45])
  c = c[::-1]  
  epsilon = 23. + 26./60. + np.polyval(c, time/1.0e2)/3600.0
  eps = ( epsilon + elong )*np.pi/180. # True obliquity in radians

  ra = np.arctan2( np.sin(lamb)*np.cos(eps) - np.tan(beta)*np.sin(eps), np.cos(lamb) )
  ra = cirrange(ra, radians=True) 
  dec = np.arcsin( np.sin(beta)*np.cos(eps) + np.cos(beta)*np.sin(eps)*np.sin(lamb) )

  if not radian:
    ra = ra/(np.pi/180.)
    dec = dec/(np.pi/180.)
  else:
    geolong = lamb
    geolat = beta

  return ra, dec, dis, geolong, geolat



from __future__ import print_function
from __future__ import division
import six.moves as smo
import numpy as np
import re
import os
from PyAstronomy.pyaC import pyaErrors as PE
from PyAstronomy.pyaC import pyaPermanent as PP
import scipy.interpolate as sci


class TransmissionCurves:
    r"""
    Photometric transmission curves for various photometric bands.

    A list of supported bands can be obtained by calling `availableBands`.

    Parameters
    ----------
    fn : string, optional
        The filename of the data file. Default is 'default',
        which means that the shipped data file will be used.
    """

    def _createSet(self, bn, lines):
        r"""
        Create data set from band name and set of lines.

        Parameters
        ----------
        bn : string
            Name of the band
        lines : list of string
            The data (table) specifying transmission as
            a function of wavelength.
        """
        if bn in self.bands:
            PE.warn(
                PE.PyAValError(
                    "Band named '" + str(bn) + "' already exists.",
                    where="TransmissionCurves (reading data from file)",
                    solution="Use unique band names.",
                )
            )
        self.bands[bn] = np.zeros((len(lines), 2))
        for i in smo.range(len(lines)):
            self.bands[bn][i, ::] = np.array(lines[i].split(), dtype=float)

    def _readData(self, fn):
        r"""
        Read data file.

        The file must introduce each band via a row like
        "# BAND: 'band-name'". Otherwise, rows preceded by
        a hash will be ignored. The data must be specified as
        a table with columns wavelength [A] and transmission (0-1)
        below the line introducing the band; an arbitrary number
        of comment lines is allowed at all positions.

        Parameters
        ----------
        fn : string
            The filename of the data file.
        """
        if not os.path.isfile(fn):
            raise (
                PE.PyAValError(
                    "No such file: " + fn,
                    where="TransmissionCurves",
                    solution="Use valid file or use 'default'.",
                )
            )
        self.bands = {}
        cb = None
        lcol = None
        for l in open(fn):
            r = re.match(r"^\s*#\s*BAND:\s+'([^']+)'.*", l)
            if not r is None:
                # New current band
                # Add the previous data set
                if not cb is None:
                    self._createSet(cb, lcol)
                # Prepare collecting data for new band
                cb = r.group(1)
                lcol = []
            else:
                if l.find("#") != -1:
                    # It is a comment not specifying a band
                    # Ignore this...
                    continue
                elif len(l.rstrip("\n")) == 0:
                    # Ignore empty lines
                    continue
                else:
                    # It is not a comment but data
                    # Collect data
                    lcol.append(l.rstrip("\n"))
        if len(lcol) > 0:
            # Add last data set
            self._createSet(cb, lcol)

    def _checkBand(self, bn):
        r"""
        Check whether band name is defined.
        """
        if not bn in self.bands:
            raise (
                PE.PyAValError(
                    "No such band: " + str(bn),
                    where="TransmissionCurves",
                    solution="Use one of: " + ", ".join(self.availableBands()),
                )
            )

    def availableBands(self):
        r"""
        Band names of available transmission curves.

        Returns
        -------
        Available names : list of strings
            All bands for which data are available.
        """
        return sorted(list(self.bands))

    def getTransCurve(self, bn, ik="linear"):
        r"""
        Get a transmission curve.

        Parameters
        ----------
        bn : string
            Name of the band.
        ik : string, optional
            The type of interpolation. Accepts all values also
            accepted by the `kind` keyword of scipy's `interp1d` routine.
            Default is 'linear'.

        Returns
        -------
        Transmission curve : callable
            An object (scipy.interpolate.interp1d) that can be called
            with wavelength (float or array in [A]) as argument and returns
            the transmission.
        """
        self._checkBand(bn)
        fi = sci.interp1d(
            self.bands[bn][::, 0],
            self.bands[bn][::, 1],
            kind=ik,
            bounds_error=False,
            fill_value=0.0,
        )
        return fi

    def getTransCurveData(self, bn):
        r"""
        Get data specifying the transmission curve.

        Returns
        -------
        Transmission table : 2d array
            A table (array) with wavelength [A] in first column and
            transmission (0-1) in the second column.
        """
        self._checkBand(bn)
        return self.bands[bn]

    def convolveWith(self, wvl, spec, bn, ik="linear"):
        r"""
        Convolve spectrum with transmission curve.

        Parameters
        ----------
        wvl, spec : arrays
            Wavelength axis [A] and spectral data.
        bn : string
            Name of the band.
        ik : string, optional
            The type of interpolation. Accepts all values also
            accepted by the `kind` keyword of scipy's `interp1d` routine.
            Default is 'linear'.

        Returns
        -------
        Convolved spectrum : array
            Input spectrum multiplied with transmission curve
            of the specified band.
        """
        self._checkBand(bn)
        tc = self.getTransCurve(bn, ik)
        return spec * tc(wvl)

    def addPassband(self, name, wvl, trans, snc=False):
        r"""
        Add a new passband to the inventory.

        Parameters
        ----------
        name : string
            The name of the passband.
        wvl : array
            Wavelength in A.
        trans : array
            Transmission of the passband.
        snc : boolean, optional
            A `Skip Name Check` flag. If False (default), an
            exception is raised if the passband name is already
            present in the inventory of passbands. Otherwise
            the old passband is replaced by the new specification.
        """
        if not snc:
            if name in self.bands:
                raise (
                    PE.PyANameClash(
                        "A passband with name '" +
                        str(name) + "' is already present.",
                        solution=[
                            "Change the name.",
                            "Use `snc=True` to ignore and overwrite old passband.",
                        ],
                    )
                )
        self.bands[name] = np.vstack((wvl, trans)).transpose()

    def addSpitzerIRACPassbands(self, forceDownload=False, verbose=True):
        r"""
        Adds Spitzer IRAC passbands.

        On first call, the passband files are downloaded.
        The files are downloaded from:

        http://irsa.ipac.caltech.edu/data/SPITZER/docs/irac/calibrationfiles/spectralresponse/

        Parameters
        ----------
        forceDownload : boolean, optional
            If True, a re-download of the passband files is triggered.
            Default is False.
        verbose : boolean, optional
            If True (default), download process will print information
            on progress.
        """

        fns = [
            "080924ch1trans_full.txt",
            "080924ch2trans_full.txt",
            "080924ch3trans_full.txt",
            "080924ch4trans_full.txt",
            "080924ch1trans_sub.txt",
            "080924ch2trans_sub.txt",
            "080924ch3trans_sub.txt",
            "080924ch4trans_sub.txt",
        ]

        path = "pyasl/resBased/"

        for fn in fns:
            fno = path + fn
            if (not self._fs.fileExists(fn)) or forceDownload:
                self._fs.downloadToFile(
                    "http://irsa.ipac.caltech.edu/data/SPITZER/docs/irac/calibrationfiles/spectralresponse/"
                    + fn,
                    fno,
                    forceDownload,
                    verbose,
                )

            dat = np.loadtxt(self._fs.requestFile(fno))
            dat[::, 0] *= 1e4

            wl = re.match(
                r".*IRAC\s+(\d+\.\d+)\s+.*", self._fs.requestFile(fno).readline()
            ).group(1)
            if fn.find("_full") != -1:
                win = "_full"
            else:
                win = "_sub"

            bn = "IRAC" + wl + win
            self.addPassband(bn, dat[::, 0], dat[::, 1], snc=True)

    def addKeplerPassband(self, forceDownload=False, verbose=True):
        r"""
        Adds Kepler mission passband.

        Kepler high-resolution passband is downloaded from:
        https://keplergo.github.io/KeplerScienceWebsite/data/kepler_response_hires1.txt

        and added as 'Kepler_HR'.

        Parameters
        ----------
        forceDownload : boolean, optional
            If True, a re-download of the passband files is triggered.
            Default is False.
        verbose : boolean, optional
            If True (default), download process will print information
            on progress.
        """

        fno = os.path.join("pyasl", "resBased",
                           "kepler_response_hires1.txt.gz")

        if (not self._fs.fileExists(fno)) or forceDownload:
            self._fs.downloadToFile(
                "https://keplergo.github.io/KeplerScienceWebsite/data/kepler_response_hires1.txt",
                fno,
                forceDownload,
                verbose,
            )

        dat = np.loadtxt(self._fs.requestFile(fno))
        # Convert into A
        dat[::, 0] *= 10.0

        bn = "Kepler_HR"
        self.addPassband(bn, dat[::, 0], dat[::, 1], snc=True)

    def addTESSPassband(self, forceDownload=False, verbose=True):
        r"""
        Adds TESS mission passband.

        TESS passband is downloaded from:
        https://heasarc.gsfc.nasa.gov/docs/tess/data/tess-response-function-v1.0.csv

        and added as 'TESS'.

        .. note:: After version 0.24.0, the TESS passband is no longer downloaded but has been integrated into PyAstronomy.

        Parameters
        ----------
        forceDownload : boolean, optional
            If True, a re-download of the passband files is triggered.
            Default is False.
        verbose : boolean, optional
            If True (default), download process will print information
            on progress.
        """
        pass

    def __init__(self, fn="default"):
        self._fs = PP.PyAFS()
        self._readData(os.path.join(
            os.path.dirname(__file__), "transCurves.dat"))



import datetime
import re
from . import pyaErrors as PE
import six

class SimIOF:
  """
    Simple Input/Output file.
    
    If a file is opened for writing, the origin and
    date will be added at the top of the file. If it
    is opened for reading, the properties will be read,
    converted into float of possible, and stored in the
    `args` attribute. If the float-conversion fails,
    the value is kept as a string.
    
    Parameters
    ----------
    origin : str
        Identifies the script/program writing the file.
    args : tuple
        Passed to the constructor of a file object.
    kwargs : dictionary
        Passed to the constructor of a file object.
  """ 
  
  def close(self):
    """
      Close file.
    """
    self.file.close()
  
  def write(self, *args, **kwargs):
    """
      Write to file object
    """
    self.file.write(*args, **kwargs)
  
  def __init__(self, origin, *args, **kwargs):
    self.origin = origin
    self.file = open(*args, **kwargs)
    if self.file.mode in ['w', 'wt']:
      self.write("# File created by " + str(origin) + "\n")
      self.write("# Date: " + str(datetime.datetime.now()).split('.')[0] + "\n")
      self.write("#\n")
    elif self.file.mode in ['r', 'rt']:
      self._readProps()

  def __del__(self):
    self.file.close()

  def _readProps(self):
    """
      Reads the properties from input file.
    """
    self.args = {}
    for l in self.file:
      r = re.match(r"^#\s+([^:]+):\s*(.*)$", l)
      if r is not None:
        p = r.group(2)
        try:
          p = float(p)
        except ValueError:
          pass
        self.args[r.group(1)] = p
    self.file.seek(0)

  def _addOneProp(self, n, v, fmt=None):
    """
      Adds a property to an output file.
      
      Parameters
      ----------
      n : string
          Name of the property
      v : any
          Value of that property
      fmt : string, optional
          The format-string used to represent the property.
    """
    if fmt is None:
      self.write("# " + str(n) + ": " + str(v) + "\n")
    else:
      if fmt.startswith("%"):
        self.write("# " + str(n) + ((": " + fmt) % v) + "\n")
      else:
        self.write("# " + str(n) + ((": " + "%" + fmt) % v) + "\n")

  def addProp(self, name, value, fmt=None):
    """
      Add a property to the file.
      
      Parameters
      ----------
      name : string or list of strings
          Name of the property
      value : any, or list of any type
          Value of that property
      fmt : string, list, optional
          The format-string used to represent the property.
          If a single string is given, it is applied to all
          properties. Otherwise, one format string for every
          property should be given. 
    """
    if isinstance(name, six.string_types):
      # There is only one value to be added
      self._addOneProp(name, value, fmt=fmt)
    else:
      # There is more than one
      for i, (n, v) in enumerate(zip(name, value)):
        if (fmt is None) or isinstance(fmt, six.string_types):
          # There is only no or one format string
          self._addOneProp(n, v, fmt=fmt)
        else:
          # There is a list of formats
          self._addOneProp(n, v, fmt=fmt[i])
          
  def addColInfo(self, cns, oneLine=True):
    """
      Add enumerated column names to file.
      
      Parameters
      ----------
      cns : list of strings
          List of names
      oneLine : boolean, optional
          If True, all definitions will be written into
          a single line. Otherwise each definition is written
          on its own line.
    """
    self.write("# ")
    for i, c in enumerate(cns, 1):
      self.write(str(i) + ") " + c)
      if oneLine and (i < len(cns)):
        self.write(", ")
      else:
        self.write("\n# ")
    self.write("\n")
    
  def __getattr__(self, attr):
    return getattr(self.file, attr)
  
  def __iter__(self):
    return iter(self.file)
  



# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from numpy import sqrt, sin, cos, pi, exp
from numpy import arcsin as asin
from numpy import arccos as acos
from scipy.optimize import fsolve
from scipy.special import i0, i1, erf
from PyAstronomy.funcFit import OneDFit, turnIntoRebin
from PyAstronomy.modelSuite.XTran import palTrans
from PyAstronomy import pyasl


class RmcL(OneDFit):
    """
    Analytical Rossiter-McLaughlin effect.

    This class implements the analytical model radial velocity (RV) curves
    for the Rossiter-McLaughlin effect given by *Ohta et. al 2005*.

    *Fit parameters*:
     - epsilon - linear limb dark
     - gamma   - Rp/Rs (ratio of planetary and stellar radius)
     - P       - Orbital period [d]
     - T0      - Central transit time
     - i       - Inclination of orbit [rad]
     - Is      - Inclination of stellar rotation axis [rad]
     - Omega   - Angular rotation velocity (star) [rad/s]
     - lambda  - Sky-projected angle between stellar
       rotation axis and normal of orbit plane [rad]
     - a       - Semi major axis [stellar radii]

    By default all parameters remain frozen.

    .. note::
        According to the input parameter units, the units of the model RV
        curve are **stellar-radii per second**.
    """

    def __init__(self):
        OneDFit.__init__(self, ["epsilon", "gamma", "P",
                                "T0", "i", "Is", "Omega", "lambda", "a"])
        self.setRootName("Ohta05")

    def planetDistance(self, f):
        return self["a"]

    def W1(self, rho):
        result = sqrt(1.0 - rho**2) - self["gamma"]**2 * \
            (2.0 - rho**2) / (8.0 * (1.0 - rho**2)**(3.0 / 2.0))
        return result

    def W2(self, rho):
        result = sqrt(1.0 - rho**2) - self["gamma"]**2 * (
            4.0 - 3.0 * rho**2) / (8.0 * (1.0 - rho**2)**(3.0 / 2.0))
        return result

    def XpVec(self, f):
        result = np.zeros(3)
        result[0] = -cos(self["lambda"]) * sin(f) - \
            sin(self["lambda"]) * cos(self["i"]) * cos(f)
        result[1] = sin(self["i"]) * cos(f)
        result[2] = sin(self["lambda"] * sin(f)) - \
            cos(self["lambda"]) * cos(self["i"]) * cos(f)
        result *= self.rp(f)
        return result

    def Xp(self, f):
        result = -cos(self["lambda"]) * sin(f) - \
            sin(self["lambda"]) * cos(self["i"]) * cos(f)
        result *= self.planetDistance(f)
        return result

    def Zp(self, f):
        result = sin(self["lambda"]) * sin(f) - \
            cos(self["lambda"]) * cos(self["i"]) * cos(f)
        result *= self.planetDistance(f)
        return result

    def etap(self, Xp, Zp):
        return sqrt(Xp**2 + Zp**2) - 1.0

    def zeta(self, etap):
        result = (2.0 * etap + self["gamma"]**2 +
                  etap**2) / (2.0 * (1.0 + etap))
        return result

    def rhoFromVec(self, XpVec):
        return sqrt(XpVec[0]**2 + XpVec[2]**2)

    def rho(self, Xp, Zp):
        return sqrt(Xp**2 + Zp**2)

    def trueAnomaly(self, time):
        result = ((time - self["T0"]) / self["P"] -
                  np.floor((time - self["T0"]) / self["P"])) * 2.0 * np.pi
        return result

    def z0(self, etap, indi):
        result = np.zeros(etap.size)
        result[indi] = sqrt((self["gamma"]**2 - etap[indi]**2) * ((etap[indi] + 2.0)**2 - self["gamma"]**2)) / \
            (2.0 * (1.0 + etap[indi]))
        return result

    def x0(self, etap):
        return 1.0 - (self["gamma"]**2 - etap**2) / (2.0 * (1.0 + etap))

    def g(self, x, e, g, x0):
        result = (1.0 - x**2) * asin(sqrt((g**2 - (x - 1.0 - e)**2) / (1.0 - x**2))) + \
            sqrt(2.0 * (1.0 + e) * (x0 - x) * (g**2 - (x - 1.0 - e)**2))
        return result

    def xc(self, zeta, x0):
        return x0 + (zeta - self["gamma"]) / 2.0

    def W3(self, x0, zeta, xc, etap):
        result = pi / 6.0 * (1.0 - x0)**2 * (2.0 + x0) + \
            pi / 2.0 * self["gamma"] * (self["gamma"] - zeta) * \
            self.g(xc, etap, self["gamma"], x0) / self.g((1.0 - self["gamma"]), -self["gamma"], self["gamma"], x0) * \
            self.W1(1.0 - self["gamma"])
        return result

    def W4(self, x0, zeta, xc, etap):
        result = pi / 8. * (1.0 - x0)**2 * (1.0 + x0)**2 + \
            pi / 2. * self["gamma"] * (self["gamma"] - zeta) * xc * \
            self.g(xc, etap, self["gamma"], x0) / self.g((1.0 - self["gamma"]), -self["gamma"], self["gamma"], x0) * \
            self.W2(1.0 - self["gamma"])
        return result

    def evaluate(self, xOrig):
        """
        Calculates and returns RV curve according to current model parameters.

        .. note:: The units of the model RV curve are **stellar-radii per second**.

        Parameters
        ----------
        xOrig : array
            The time stamps at which to calculate the model RV curve.
            Note that the orbit period and central transit time are used
            to convert time into "true anomaly".
        """
        x = self.trueAnomaly(xOrig)
        Xp = self.Xp(x)
        Zp = self.Zp(x)
        rho = self.rho(Xp, Zp)
        etap = self.etap(Xp, Zp)
        zeta = self.zeta(etap)
        x0 = self.x0(etap)
        xc = self.xc(zeta, x0)

        # dphase is the phase difference between the primary transit and the time points
        # It is used to exclude the secondary transit from the calculations
        dphase = np.abs((xOrig - self["T0"]) / self["P"])
        dphase = np.minimum(dphase - np.floor(dphase),
                               np.abs(dphase - np.floor(dphase) - 1))

        y = np.zeros(len(x))
        indi = np.where(np.logical_and(
            rho < (1.0 - self["gamma"]), dphase < 0.25))[0]

        y[indi] = Xp[indi] * self["Omega"] * sin(self["Is"]) * self["gamma"]**2 * \
            (1.0 - self["epsilon"] * (1.0 - self.W2(rho[indi]))) / \
            (1.0 - self["gamma"]**2 - self["epsilon"] * (1. /
                                                         3. - self["gamma"]**2 * (1.0 - self.W1(rho[indi]))))

        indi = np.where(np.logical_and(
            np.logical_and(rho >= 1. - self["gamma"], rho < 1.0 + self["gamma"]), dphase < 0.25))[0]
        z0 = self.z0(etap, indi)

        y[indi] = (Xp[indi] * self["Omega"] * sin(self["Is"]) * (
            (1.0 - self["epsilon"]) * (-z0[indi] * zeta[indi] + self["gamma"]**2 * acos(zeta[indi] / self["gamma"])) +
            (self["epsilon"] / (1.0 + etap[indi])) * self.W4(x0[indi], zeta[indi], xc[indi], etap[indi]))) / \
            (pi * (1. - 1.0 / 3.0 * self["epsilon"]) - (1.0 - self["epsilon"]) * (asin(z0[indi]) - (1. + etap[indi]) * z0[indi] +
                                                                                  self["gamma"]**2 * acos(zeta[indi] / self["gamma"])) - self["epsilon"] * self.W3(x0[indi], zeta[indi], xc[indi], etap[indi]))

        return y


RmcL_Rebin = turnIntoRebin(RmcL)



class RmcLell(OneDFit):
    """
    Analytical Rossiter-McLaughlin effect.

    This class implements the analytical model radial velocity (RV) curves
    for the Rossiter-McLaughlin effect given by *Ohta et. al 2005*.

    *Fit parameters*:
     - epsilon - Linear limb darkening coefficient
     - gamma   - Rp/Rs (ratio of planetary and stellar radius)
     - P       - Orbital period [d]
     - tau     - Time of periastron passage [same as orbital period]
     - i       - Inclination of orbit [rad]
     - Is      - Inclination of stellar rotation axis [rad]
     - Omega   - Angular rotation velocity (star) [rad/s]
     - lambda  - Sky-projected angle between stellar
                 rotation axis and normal of orbit plane [rad]
     - a       - Semi major axis [stellar radii]
     - w       - Argument of periapsis [rad]
     - e       - Eccentricity

    .. note::
        In the case of zero eccentricity, a value of -90 deg for the argument of
        periastron (w) makes the time of periastron (tau) numerically identical
        with the central transit time of the circular case (T0).

    .. note::
         According to the input parameter units, the units of the model RV
         curve are **stellar-radii per second**.
    """

    def __init__(self):
        OneDFit.__init__(self, ["epsilon", "gamma", "P",
                                "tau", "i", "Is", "Omega", "lambda", "a", "e", "w"])
        self.setRootName("OhtaELL05")
        self["P"] = 1
        self._ke = pyasl.KeplerEllipse(1, 1)
        self._setkepellpars()

    def _setkepellpars(self):
        """ Set parameters values of Kepler ellipse object from current settings """
        self._ke.i = self["i"]/np.pi*180
        self._ke.w = self["w"]/np.pi*180
        self._ke.e = self["e"]
        self._ke.a = self["a"]
        self._ke.per = self["P"]
        self._ke.tau = self["tau"]
        self._ke.Omega = self["lambda"]/np.pi*180

    g = RmcL.g
    W1 = RmcL.W1
    W2 = RmcL.W2
    W3 = RmcL.W3
    W4 = RmcL.W4

    def evaluate(self, xOrig):
        """
        Calculates and returns RV curve according to current model parameters.

        .. note:: The units of the model RV curve are **stellar-radii per second**.

        Parameters
        ----------
        xOrig : array
            The time stamps at which to calculate the model RV curve.
            Note that the orbit period and central transit time are used
            to convert time into "true anomaly".
        """
        self._setkepellpars()
        # In coordinate system with observer in -z axis
        pos = self._ke.xyzPos(xOrig)
        # Use coordinate used by Ohta (looking into -y direction)
        Xp = -pos[::,0]
        Zp = pos[::,1]
        Yp = pos[::,2]

        rho = RmcL.rho(self, Xp, Zp)
        etap = RmcL.etap(self, Xp, Zp)
        zeta = RmcL.zeta(self, etap)
        x0 = RmcL.x0(self, etap)
        xc = RmcL.xc(self, zeta, x0)

        # Planet in front of star
        y = np.zeros_like(xOrig)
        indi = np.where((Yp < 0) & (rho < (1.0 - self["gamma"])))[0]

        y[indi] = Xp[indi] * self["Omega"] * sin(self["Is"]) * self["gamma"]**2 * \
            (1.0 - self["epsilon"] * (1.0 - RmcL.W2(self, rho[indi]))) / \
            (1.0 - self["gamma"]**2 - self["epsilon"] * (1. /
                                                         3. - self["gamma"]**2 * (1.0 - RmcL.W1(self, rho[indi]))))

        indi = np.where((rho >= 1. - self["gamma"]) & (rho < 1.0 + self["gamma"]) & (Yp < 0))[0]
        z0 = RmcL.z0(self, etap, indi)

        y[indi] = (Xp[indi] * self["Omega"] * sin(self["Is"]) * (
            (1.0 - self["epsilon"]) * (-z0[indi] * zeta[indi] + self["gamma"]**2 * acos(zeta[indi] / self["gamma"])) +
            (self["epsilon"] / (1.0 + etap[indi])) * RmcL.W4(self, x0[indi], zeta[indi], xc[indi], etap[indi]))) / \
            (pi * (1. - 1.0 / 3.0 * self["epsilon"]) - (1.0 - self["epsilon"]) * (asin(z0[indi]) - (1. + etap[indi]) * z0[indi] +
                                                                                  self["gamma"]**2 * acos(zeta[indi] / self["gamma"])) - \
                                                                                  self["epsilon"] * RmcL.W3(self, x0[indi], zeta[indi], xc[indi], etap[indi]))

        return y



class RmcL_Hirano(OneDFit):

    def __init__(self):
        """
        This class implements analytical expressions for the Rossiter-McLaughlin \
        effect for the case when the anomalous radial velocity is obtained by \
        cross-correlation with a stellar spectrum, according to *Hirano et. al 2010*.

        .. note::
          The intrinsic line profile and the rotation kernel are both \
          approximated by Gaussians, while the planet is assumed to be \
          'sufficiently small enough'.

        *Fit parameters*:
         - linLimb - linear limb-darkening parameter
         - quadLimb - quadratic limb-darkening parameter
         - gamma   - Rp/Rs (ratio of planetary and stellar radius)
         - P       - Orbital period [d]
         - T0      - Central transit time
         - i       - Inclination of orbit [rad]
         - Is      - Inclination of stellar rotation axis [rad]
         - Omega   - Angular rotation velocity (star) [rad/s]
         - lambda  - Sky-projected angle between stellar rotation axis and normal of orbit plane [rad]
         - a       - Semi major axis [stellar radii]
         - vbeta   - natural line broadening [km/s]
         - vSurf   - maximum surface velocity [km/s]

       By default all parameters remain frozen.

       .. note::
         According to the input parameter units, the units of the model RV curve are **stellar-radii per second**.
         The original formulation by Hirano et al. uses two dispersion parameters, \
         `beta` and `sigma`. `beta` describes the width of an intrinsic line profile \
         modeled by a Gaussian; it corresponds to a 'broadening velocity' \
         vbeta = beta * c / lam0, where lam0 denotes a typical wavelength scale. \
         The dispersion parameter `sigma` describes line broadening due to stellar \
         rotation; it corresponds to a velocity of vSurf*sin(Is) = alpha*sigma*c/lam0, \
         where lam0 denotes the typical wavelength and alpha is a scaling \
         parameters depending on the limb-darkening coefficients.

        """
        OneDFit.__init__(self, ["linLimb", "quadLimb", "gamma", "P",
                                "T0", "i", "Is", "Omega", "lambda", "a", "vbeta", "vSurf"])
        self.flux = None
        self.setRootName("Hira10")

    def planetDistance(self, f):
        return self["a"]

    def Xp(self, f):
        result = -cos(self["lambda"]) * sin(f) - \
            sin(self["lambda"]) * cos(self["i"]) * cos(f)
        result *= self.planetDistance(f)
        return result

    def trueAnomaly(self, time):
        result = ((time - self["T0"]) / self["P"] -
                  np.floor((time - self["T0"]) / self["P"])) * 2.0 * np.pi
        return result

    def supplyFlux(self, flux):
        """
        This method can be used to supply own simultaneously obtained photometric data.
        The continuum flux should be normalized to one.
        """
        self.flux = flux
        return 0

    def EqF6(self, x):
        """
        Relation between rotation and Gaussian broadening kernels by least-squares fitting
        """
        result = 1. / sqrt(2.) - (12. * (self["linLimb"] + 2. * self["quadLimb"]) * exp(-x**2)) / (x**2 * (-6. + 2. * self["linLimb"] + self["quadLimb"])) \
            + (6. * exp(-x**2 / 2.)) / (x**3 * (-6. + 2. * self["linLimb"] + self["quadLimb"])) * (-2. * x**3 * (-1. + self["linLimb"] + self["quadLimb"]) * i0(x**2 / 2.)
                                                                                                   + 2. * x *
                                                                                                   (-2. * self["quadLimb"] + x**2 * (-1. + self["linLimb"] + self["quadLimb"])) * i1(
                                                                                                       x**2 / 2.)
                                                                                                   + sqrt(pi) * exp(x**2 / 2.) * (self["linLimb"] + 2. * self["quadLimb"]) * erf(x))
        return result

    def evaluate(self, time):
        """
        Calculates and returns RV curve according to current model parameters.

        .. note:: The units of the model RV curve are **stellar-radii per second**.

        Parameters
        ----------
        time : array
            The time stamps at which to calculate the model RV curve.
            Note that the orbit period and central transit time are
            used to convert time into "true anomaly".
        """
        x = self.trueAnomaly(time)
        Xp = self.Xp(x)

        if self.flux != None:
            f = 1. - self.flux
        else:
            plc = palTrans.PalLC()
            plc.assignValue({"p": self["gamma"],
                             "per": self["P"],
                             "a": self["a"],
                             "i": self["i"] * 180. / np.pi,
                             "linLimb": self["linLimb"],
                             "quadLimb": self["quadLimb"],
                             "T0": self["T0"],
                             "b": 0.0})
            f = 1. - plc.evaluate(time)

        deltaV = Xp * self["Omega"] * sin(self["Is"]) * f

        # Calculate scaling factor alpha, Eq. F6 in Hirano et al. 2010
        alpha = fsolve(self.EqF6, x0=1.0, xtol=1e-8)[0]

        # Determine effective rotational velocity
        vsiniEff = self["vSurf"] * sin(self["Is"]) / alpha

        # Apply the Hirano et al. 2010 correction factor, Eq. 37 in Hirano et al. 2010
        deltaV = deltaV * ((2. * self["vbeta"]**2 + 2. * vsiniEff**2) / (2. * self["vbeta"]**2 + vsiniEff**2))**(
            3. / 2.) * (1. - (Xp * self["Omega"] * sin(self["Is"]))**2 / (2. * self["vbeta"]**2 + vsiniEff**2))
        return deltaV



import matplotlib
# matplotlib.use('TkAgg')

from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as NavigationToolbar2TkAgg
from matplotlib.figure import Figure
from PyAstronomy.pyaC import pyaErrors as PE
from PyAstronomy import funcFit as fuf
import numpy as np

import six.moves.tkinter as tk
import six.moves as smo
import six


class Point():
  
  def __init__(self, event=None):
    self.xdata = 0.0
    self.ydata = 0.0
    self.mplLine = None
    self.lbIdent = None
    if event is not None:
      self.xdata = event.xdata
      self.ydata = event.ydata
  
  def asStr(self):
    """
      String representation of the point.
      
      Returns
      -------
      Point : string
          String representation of the point coordinates.
    """
    return "%g, %g" % (self.xdata, self.ydata)


class IAGVFit:
  """
    GUI for an interactive of spectral lines.
    
    Parameters
    ----------
    x, y : array
        The data (e.g., spectrum).
    yerr : array, optional
        The error on the data.
    mode : string, {voigt,gauss}, optional
        The line profile to be used. Either a
        Gaussian or a Voigt profile can be used.
    config : dictionary, optional
        A dictionary containing any combination of
        the keys, which can be used to change the default
        behavior of the class.

        +------------------------+------------+---------------------------------+
        | Key                    | Default    | Description                     |
        +========================+============+=================================+
        | modelLS                | r--        | Style used for the combined     |
        |                        |            | model.                          |
        +------------------------+------------+---------------------------------+
        | dataLS                 | bp         | Style used for the data of no   |
        |                        |            | error is given.                 |
        +------------------------+------------+---------------------------------+
        | dataLSerr              | b+         | Style used when error is given. |
        +------------------------+------------+---------------------------------+
        | activeCompLS           | k-         | Style used to plot active       |
        |                        |            | component.                      |
        +------------------------+------------+---------------------------------+
        | inactiveCompLS         | y--        | Style used to plot inactive     |
        |                        |            | component(s).                   |
        +------------------------+------------+---------------------------------+
        | plotIndComps           | True       | If True, individual components  |
        |                        |            | are plotted.                    |
        +------------------------+------------+---------------------------------+
        | rangeLS                | m--        | Style used to plot lines        |
        |                        |            | indicating the fitting range.   |
        +------------------------+------------+---------------------------------+
        | defaultFreeVoigt       | A, ad, al, | Default free fitting parameters |
        |                        | mu         | for Voigt profile.              |
        +------------------------+------------+---------------------------------+
        | default_alad           | 0.1        | Default ratio between width of  |
        |                        |            | Lorentz and Gauss contribution. |
        +------------------------+------------+---------------------------------+
        | defaultFreeGauss       | A, ad, sig | Default set of free fitting     |
        |                        |            | parameters for Gaussian.        |
        +------------------------+------------+---------------------------------+
        | defaultWheelAdd        | 0.01       | Default increment for additive  |
        |                        |            | mouse wheel manipulation.       |
        +------------------------+------------+---------------------------------+
        | defaultWheelMult       | 2          | Default factor for              |
        |                        |            | multiplicative mouse wheel      |
        |                        |            | manipulation.                   |
        +------------------------+------------+---------------------------------+
                    
    Attributes
    ----------
    f : mpl Figure
        A Figure instance from matplotlib.
    a : mpl axis
        An Axis instance from matplotlib.
  """
  
  def __init__(self, x, y, yerr=None, mode="gauss", config=None):
    self.windowTitle = "PyA interactive GV"
    self.f = Figure()
    self.a = self.f.add_subplot(111)
        
    self._config = {"modelLS":'r--', \
                    "dataLS":'bp', \
                    "dataLSerr":'b+', \
                    "activeCompLS":'k-', \
                    "inactiveCompLS":'y--', \
                    "plotIndComps":True, \
                    "rangeLS":'m--', \
                    "defaultFreeVoigt":["A", "ad", "al", "mu"], \
                    "default_alad":0.1, \
                    "defaultFreeGauss":["A", "sig", "mu"], \
                    "defaultWheelAdd":0.01, \
                    "defaultWheelMult": 2}
    
    # Modify configuration according to input
    if config is not None:
      for k in six.iterkeys(self._config):
        if k in config:
          # Value has been modified
          self._config[k] = config[k]
    
    # Tk root
    self.root = tk.Tk()
    
    # Create data plot
    self._x = x
    self._y = y
    self._yerr = yerr
    if self._yerr is None:
      self.a.plot(x, y, self._config["dataLS"])
    else:
      self.a.errorbar(x, y, yerr=self._yerr, fmt=self._config["dataLSerr"])
    
    # The input mode determines how clicks on the mouse wheel are
    # interpreted.
    self._inputMode = "newComp"
     
    # Component count/assignment starts at 1
    self._activeComponent = 1
    self._compoCount = 0
    
    # Create model and parameter lists
    self._mode = mode
    if self._mode == "voigt":
      self._model = fuf.MultiVoigt1d(1)
      self._compPars = ["mu", "ad", "al", "A"]
    elif self._mode == "gauss":
      self._model = fuf.MultiGauss1d(1)
      self._compPars = ["mu", "sig", "A"]
    else:
      raise(PE.PyAValError("No such mode: " + str(self._mode), \
                           solution="Choose either 'voigt' or 'gauss'."))
    self._model["off"] = 1.0

    # List used to cache points used to add component
    # as long as component is not completely specified
    self._pointCache = []

    # What to consider in the fit
    self._rangeIndices = np.arange(self._x.size, dtype=int)
    
    # A frame containing the mpl plot
    self.plotFrame = tk.Frame()
    self.plotFrame.pack(fill=tk.BOTH, side=tk.LEFT, expand=True)
    self.canvas = FigureCanvasTkAgg(self.f, master=self.plotFrame)
    
    # A frame containing the box with selected points
    # and control buttons
    self.pointFrame = tk.Frame(self.root)
    self.pointFrame.pack(side=tk.LEFT, fill=tk.BOTH)
    
    # Manage active component
    self.activeCompoFrame = tk.Frame(self.pointFrame)
    self.activeCompoFrame.pack(side=tk.TOP)
    self.currentComponentLabel = tk.Label(self.activeCompoFrame, text="Active component")
    self.currentComponentLabel.pack(side=tk.TOP)
    # De- and increase no. of active component
    self.downACB = tk.Button(self.activeCompoFrame, text="-", command=self._downAC)
    self._acstring = tk.StringVar()
    self._acstring.set(str(self._activeComponent)) 
    self.ACLabel = tk.Label(self.activeCompoFrame, textvariable=self._acstring)
    self.upACB = tk.Button(self.activeCompoFrame, text="+", command=self._upAC)
    self.downACB.pack(side=tk.LEFT)
    self.ACLabel.pack(side=tk.LEFT)
    self.upACB.pack(side=tk.RIGHT)
    
    # Show and work with parameters
    self.parameterFrame = tk.Frame(self.pointFrame, relief=tk.RAISED, borderwidth=1)
    self.parameterFrame.pack(side=tk.TOP)
    # Headline
    tk.Label(self.parameterFrame, text="Parameters (active comp.)").pack(side=tk.TOP)
    # List of parameters to show
    pars = ["off"]
    pars.extend(self._compPars)
    self._parSelect = tk.IntVar()
    # Frames for individual parameters
    self._parFrames = []
    self._parValues = []
    self._parFreeVars = []
    
    for i, p in enumerate(pars):
      self._parFrames.append(tk.Frame(self.parameterFrame))
      self._parFrames[-1].pack(side=tk.TOP)
      b = tk.Radiobutton(self._parFrames[-1], text="", variable=self._parSelect, value=i)
      b.pack(side=tk.LEFT, anchor=tk.W)
      l = tk.Label(self._parFrames[-1], text=("%3s" % p), width=3)
      l.pack(side=tk.LEFT, anchor=tk.W)
      self._parValues.append(tk.StringVar())
      self._parValues[-1].set("0.0")
      l = tk.Label(self._parFrames[-1], textvariable=self._parValues[-1], width=10)
      l.pack(side=tk.LEFT)
      
      self._parFreeVars.append(tk.IntVar())
      c = tk.Checkbutton(self._parFrames[-1], text="free", variable=self._parFreeVars[-1], \
                         command=self._freeCheckboxClicked)
      c.pack(side=tk.LEFT)
    
    # Wheel modification
    self._wheelModFrame = tk.Frame(self.pointFrame, relief=tk.RAISED, borderwidth=1)
    self._wheelModFrame.pack(side=tk.TOP)
    # Headline
    tk.Label(self._wheelModFrame, text="Mouse wheel").pack(side=tk.TOP)
    self._multAddSelect = tk.IntVar()
    self._multAddSelect.set(1)
    multiFrame = tk.Frame(self._wheelModFrame)
    addFrame = tk.Frame(self._wheelModFrame)
    multiFrame.pack(side=tk.TOP)
    addFrame.pack(side=tk.TOP)
    tk.Radiobutton(multiFrame, text="multiply [%]", variable=self._multAddSelect, value=1, width=9, anchor=tk.W).pack(side=tk.LEFT)
    tk.Radiobutton(addFrame, text="add", variable=self._multAddSelect, value=2, width=9, anchor=tk.W).pack(side=tk.LEFT)
    self._multNumber = tk.StringVar()
    self._addNumber = tk.StringVar()
    self._multNumber.set(str(self._config["defaultWheelMult"]))
    self._addNumber.set(str(self._config["defaultWheelAdd"]))
    self._multNumberEntry = tk.Entry(multiFrame, textvariable=self._multNumber, width=10)
    self._multNumberEntry.pack(side=tk.RIGHT)
    self._addNumberEntry = tk.Entry(addFrame, textvariable=self._addNumber, width=10)
    self._addNumberEntry.pack(side=tk.RIGHT)
    
    # Common width
    if self._mode == "gauss":
      commonSigFrame = tk.Frame(self.pointFrame)
      commonSigFrame.pack(side=tk.TOP)
      self._commonSig = tk.IntVar()
      c = tk.Checkbutton(commonSigFrame, text="Common width (sig)", variable=self._commonSig, \
                         command=self._commonSigClicked)
      c.pack(side=tk.LEFT)
    
    # Fit button
    self._fitButton = tk.Button(self.pointFrame, text="Fit", command=self._fitModel)
    self._fitButton.pack(side=tk.TOP, fill=tk.X)
    
    # Range button
    self._rangeButton = tk.Button(self.pointFrame, text="Set fit range", command=self._rangeButtonClicked)
    self._rangeButton.pack(side=tk.TOP, fill=tk.X)
    
    # Remove button
    self.removeButton = tk.Button(master=self.pointFrame, text="Remove component", \
                                  command=self._removeButtonClicked)
    self.removeButton.pack(side=tk.TOP, fill=tk.X)

    # a tk.DrawingArea
    self.canvas.get_tk_widget().pack()
    self.cid = self.f.canvas.mpl_connect('button_press_event', self._mouseButtonClicked)
    self.mwe = self.f.canvas.mpl_connect('scroll_event', self._mouseWheel)
    
    self.toolbar = NavigationToolbar2TkAgg(self.canvas, self.plotFrame)
    self.toolbar.update()
    self.canvas._tkcanvas.pack(side=tk.TOP, fill=tk.BOTH, expand=True)

    def _quit():
      # stops main loop
      self.root.quit()
      # this is necessary on Windows to prevent
      # Fatal Python Error: PyEval_RestoreThread: NULL tstate
      self.root.destroy()

    self.quitButton = tk.Button(master=self.pointFrame, text='Quit', command=_quit)
    self.quitButton.pack(side=tk.BOTTOM, fill=tk.X)

  def _commonSigClicked(self):
    """
      Relate/untie the width parameter for Gaussian fit.
      
      By default, all widths are coupled to "sig1".
    """
    if self._compoCount < 2:
      return
    if self._commonSig.get() == 1:
      # Relate the widths
      for i in smo.range(1, self._compoCount+1):
        if i == 1:
          continue
        self._model.thaw("sig"+str(i))
        self._model.relate("sig"+str(i), ["sig"+str(1)])
    else:
      # Untie the widths
      after = "sig1" in self._model.freeParamNames()
      for i in smo.range(1, self._compoCount+1):
        if i == 1:
          continue
        self._model.untie("sig"+str(i), forceFree=after)

  def _mouseWheel(self, event):
    """
      React to mouse wheel
    """
    if self._compoCount < 1:
      return
    if self._parSelect.get() == 0:
      name = "off"
    else:
      name = self._compPars[self._parSelect.get()-1]
      name += str(self._activeComponent)
    
    if self._multAddSelect.get() == 1:
      # Multiplication
      try:
        if self._multNumberEntry.cget("bg") == "red":
          self._multNumberEntry.config(background=self.root.cget("bg"))
        delta = float(self._multNumber.get())
      except:
        self._multNumberEntry.config(background="red")
        return
      self._model[name] = self._model[name] * (1.0 + event.step*delta/100.0)
    else:
      # Add
      try:
        if self._addNumberEntry.cget("bg") == "red":
          self._addNumberEntry.config(background=self.root.cget("bg"))
        delta = float(self._addNumber.get())
      except:
        self._addNumberEntry.config(background="red")
        delta = self._config["defaultWheelAdd"]
      self._model[name] = self._model[name] + event.step*delta

    self._updateGUI()
    
  def _downAC(self):
    """
      Decrease no. of active component
    """
    if self._activeComponent > 1:
      self._activeComponent -= 1
    self._acstring.set(str(self._activeComponent))
    self._updateGUI()

  def _upAC(self):
    """
      Increase no. of active component
    """
    if self._activeComponent < self._compoCount:
      self._activeComponent += 1
    self._acstring.set(str(self._activeComponent))
    self._updateGUI()

  def _updateParValues(self):
    """
      Update GUI state of parameter values
    """
    if self._compoCount < 1:
      return
    self._parValues[0].set("%9.4f" % self._model["off"])
    for i, p in enumerate(self._compPars, 1):
      self._parValues[i].set("%9.4f" % (self._model[p+str(self._activeComponent)]))

  def _updateFreeCheckBox(self):
    """
      Update GUI state of free/frozen checkboxes
    """
    if self._compoCount < 1:
      return
    self._parFreeVars[0].set("off" in self._model.freeParamNames())
    for i, p in enumerate(self._compPars, 1):
      if (self._mode == "gauss") and (self._commonSig.get() == 1) and (p.find("sig") != -1):
        # Gaussian width is coupled so "sig1" is the relevant parameter.
        # Show all sigs as free of they are coupled and sig1 is free.
        self._parFreeVars[i].set("sig1" in  self._model.freeParamNames())
        continue
      self._parFreeVars[i].set((p+str(self._activeComponent)) in self._model.freeParamNames())

  def _freeCheckboxClicked(self):
    """
      Transfer GUI information to model
    """
    pars = ["off"]
    pars.extend([x+str(self._activeComponent) for x in self._compPars])
    
    if (self._mode == "gauss") and (self._commonSig.get() == 1):
      # If Gaussian widths are coupled, sig1 is the only relevant
      # parameter.
      for i, p in enumerate(pars):
        if p.find("sig") != -1:
          pars[i] = "sig1"
    
    free = []
    frozen = []
    for i, p in enumerate(pars):
      if self._parFreeVars[i].get() > 0:
        free.append(p)
      else:
        frozen.append(p)

    self._model.thaw(free)
    self._model.freeze(frozen)
  
  def _updateGUI(self):
    """
      Synchronize GUI and internal state.
    """
    self._updateModelPlot()
    self._updateView()
    self._updateParValues()
    self._updateFreeCheckBox()
    self._updateFreeCheckBox()
      
  def _fitModel(self):
    """
      Fit the model.
    """
    if self._yerr is None:
      self._model.fit(self._x[self._rangeIndices], self._y[self._rangeIndices])
    else:
      self._model.fit(self._x[self._rangeIndices], self._y[self._rangeIndices], \
                      yerr=self._yerr[self._rangeIndices])
    self._updateGUI()
    
  def _updateModelPlot(self):
    """
      Update the plot describing the current model.
    """
    # Remove existing lines
    if hasattr(self, "_modelLines"):
      for l in self._modelLines:
        self.a.lines.remove(l)
    
    self._modelLines = []
    model = self._model.evaluate(self._x)
    self.a.plot(self._x, model, self._config["modelLS"])
    self._modelLines.append(self.a.lines[-1])
    
    if self._config["plotIndComps"]:
      # Save parameters
      spars = self._model.parameters()
      # Set all areas to 0
      for i in smo.range(self._compoCount):
        n = str(i+1)
        self._model["A"+n] = 0.0
      # Add individual lines
      for i in smo.range(self._compoCount):
        n = str(i+1)
        self._model["A"+n] = spars["A"+n]
        imod = self._model.evaluate(self._x)
        # Assign line style
        if (i+1) == self._activeComponent:
          lst = self._config["activeCompLS"]
        else:
          lst = self._config["inactiveCompLS"]
        self.a.plot(self._x, imod, lst)
        self._modelLines.append(self.a.lines[-1])
        # Switch off the component again
        self._model["A"+n] = 0.0
      # Restore the old state of parameters
      self._model.assignValues(spars)
    
  def _cleanPointCache(self):
    """
      Empty point cache and remove lines from plot.
    """
    for p in self._pointCache:
      # Remove from plot
      self.a.lines.remove(p[1])
    self._pointCache = []
  
  def _clearRangeCache(self):
    """
      Empty range cache and remove any lines from plot.
    """
    for e in self._rangeCache:
      self.a.lines.remove(e[1])
    self._rangeCache = []
  
  def _rangeButtonClicked(self):
    """
      Initiate input of fit range.
    """
    if not hasattr(self, "_rangeCache"):
      self._rangeCache = []
    
    if (self._inputMode == "range") or (len(self._rangeCache) == 2):
      self._clearRangeCache()
      self._updateGUI()
    self._inputMode = "range"
  
  def _newCompAddPoint(self, event):
    """
      Add a point to point cache or create component if point set is complete.
    """
    p = Point(event)
    # Plot point on canvas
    self.a.plot([p.xdata], [p.ydata], 'r+', markersize=10)
    self._pointCache.append([p, self.a.lines[-1]])
    
    if len(self._pointCache) < 3:
      # More points have to be added 
      self._updateView()
      return
    
    # Three points have been added
    fwhm = np.abs(self._pointCache[0][0].xdata - self._pointCache[2][0].xdata)
    depth = self._model["off"] - self._pointCache[1][0].ydata
    # Estimate std (Gaussian)
    sigma = fwhm/2.35482
    # Estimate area of a Gaussian with given depth
    agauss = sigma*np.sqrt(2.*np.pi)*depth
    
    # Add further component
    self._compoCount += 1
    if self._mode == "voigt":
      nmodel = fuf.MultiVoigt1d(self._compoCount)
    elif self._mode == "gauss":
      nmodel = fuf.MultiGauss1d(self._compoCount)
    # Assign parameters from old model
    nmodel.assignValues(self._model.parameters())
    # Transfer free parameters 
    nmodel.thaw(self._model.freeParamNames())
    self._model = nmodel
    
    if self._mode == "voigt":
      c0 = 2.0056
      c1 = 1.0593
      # fl/fg
      phi = self._config["default_alad"]
      fg = fwhm/(1.0 - c0*c1 + np.sqrt(phi**2 + 2.*c1*phi + c0**2*c1**2))
      fl = phi*fg
      n = str(self._compoCount)
      self._model["A"+n] = -agauss
      self._model["al"+n] = fl/2.0
      self._model["ad"+n] = fg/(2.*(2.*np.log(2.0)))
      self._model["mu"+n] = self._pointCache[1][0].xdata
      # Thaw default parameters
      self._model.thaw([x+n for x in self._config["defaultFreeVoigt"]])
    elif self._mode == "gauss":
      n = str(self._compoCount)
      self._model["A"+n] = -agauss
      self._model["sig"+n] = sigma
      self._model["mu"+n] = self._pointCache[1][0].xdata
      # Thaw default parameters
      self._model.thaw([x+n for x in self._config["defaultFreeGauss"]])
      if (self._commonSig.get() == 1) and (self._compoCount > 1):
        # Common widths for Gaussians requested
        self._model.thaw("sig"+n)
        self._model.relate("sig"+n, ["sig"+str(1)])
    
    self._cleanPointCache()
    self._updateGUI()
    
  def _addRangePoint(self, event):
    """
      Define fit range and finalize of complete.
    """
    p = Point(event)
    # Plot point on canvas
    self.a.plot([p.xdata]*2, [self._y.min(), self._y.max()], self._config["rangeLS"])
    self._rangeCache.append([p, self.a.lines[-1]])
    self._updateGUI()
    
    if len(self._rangeCache) < 2:
      # Another point has to be added
      return
    
    # There are two points. Therefore, the range can be set.
    self._range = [self._rangeCache[0][0].xdata, self._rangeCache[1][0].xdata]
    self._rangeIndices = np.where((self._x >= self._range[0]) & (self._x <= self._range[1]))[0]
    # Reset mode
    self._inputMode = "newComp"
    
  def _mouseButtonClicked(self, event):
    """
      Called on click of mouse button.
    """
    # Accept only middle button
    if event.button != 2: return
    # Check whether click occurred on plot axes.
    if event.inaxes is not self.a: return
    
    if self._inputMode == "newComp":
      self._newCompAddPoint(event)
    elif self._inputMode == "range":
      self._addRangePoint(event)
    
  def _removeButtonClicked(self):
    """
      Remove the active component.
    """
    if self._compoCount < 2:
      return
    
    # Create new model
    if self._mode == "voigt":
      nm = fuf.MultiVoigt1d(self._compoCount-1)
    else:
      nm = fuf.MultiGauss1d(self._compoCount-1)
    
    # The offset has to be treated separately
    nm["off"] = self._model["off"]
    if "off" in self._model.freeParamNames():
      nm.thaw("off")
    
    # t counts the "target" component
    t = 0
    for i in smo.range(1, self._compoCount+1):
      if i == self._activeComponent:
        # Disregard the currently active component
        continue
      n = str(i)
      t += 1
      for p in self._compPars:
        nm[p + str(t)] = self._model[p + n]
        if (p+n) in self._model.freeParamNames():
          nm.thaw(p + str(t))

    self._model = nm
    self._compoCount -= 1
    self._downAC()
    self._updateGUI()
      
  def _updateView(self):
    """
      Redraw MPL canvas
    """
    self.f.canvas.draw()

  def interactiveFit(self):
    """
      Carry out the interactive fit
      
      Returns
      -------
      Parameters : dict
          Dictionary mapping parameter name to the value.
      Active parameters : dict
          Dictionary mapping active parameter names (not including
          any numbers) to value
      Active component : int
          Number of the 'active component'. Counting starts at one.
    """
    self.root.wm_title(self.windowTitle)
    self.canvas.show()
    tk.mainloop()
    # Prepare return value
    aps = {"off":self._model["off"]}
    n = str(self._activeComponent)
    for p in self._compPars:
      aps[p] = self._model[p+n]
    return self._model.parameters(), aps, self._activeComponent
    
    




import unittest
class TestSanityOfNotebook(unittest.TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass
        
    def testsanity_example(self):
        #!/usr/bin/env python
        # coding: utf-8
        
        # Restrictions via algorithm (fmin_l_bfgs_b)
        # =================================================
        
        # In[1]:
        
        
        from numpy import arange, sqrt, exp, pi, random, ones_like
        import matplotlib.pylab as plt
        from PyAstronomy import funcFit2 as fuf2
        import scipy.optimize as sco
        import matplotlib
        matplotlib.use('Agg')
        
        random.seed(1234)
        
        # Creating a Gaussian with some noise
        # Choose some parameters...
        gPar = {"A":-5.0, "sig":10.0, "mu":10.0, "off":1.0, "lin":0.0}
        # Calculate profile
        x = arange(100) - 50.0
        y = gPar["off"] + gPar["A"] / sqrt(2*pi*gPar["sig"]**2) \
            * exp(-(x-gPar["mu"])**2/(2*gPar["sig"]**2))
        # Add some noise...
        y += random.normal(0.0, 0.01, x.size)
        # ...and save the error bars
        yerr = ones_like(x)*0.01
        # Let us see what we have done...
        plt.plot(x, y, 'bp')
        
        # Create a model object
        gf = fuf2.GaussFit()
        
        # Set guess values for the parameters
        gf.assignValues({"A":-3, "sig":10.77, "off":0.96, "mu":10.5})
        
        # 'Thaw' those (the order is irrelevant)
        gf.thaw(["mu", "sig", "off", "A"])
        
        # We need the order to get the order of bounds right
        # This is not necessarily the order in which they are thawed!
        print("Free parameter names and their order: ", gf.freeParamNames())
        
        # Use fmin_l_bfgs_b with area restricted to the (-2,0) interval
        fr = sco.fmin_l_bfgs_b(gf.chisqr, gf.freeParamVals(), args=(x,y,yerr), \
                               bounds=((-2.,0), (None,None), (None,None), (None,None)), \
                               approx_grad=True)
        # Set the parameter values to best-fit
        gf.setFreeParamVals(fr[0])
        
        gf.parameterSummary()
        plt.title("Bad fit because of restricted parameter range")
        plt.plot(x, gf.evaluate(x), 'r--')
        plt.show()
        



from __future__ import print_function, division
import unittest
from PyAstronomy import pyasl


class TestSanityOfPyaslExt1(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testsanity_pizzolatoExample(self):
        """
        Example of Pizzolato 2003 relations
        """
        from PyAstronomy import pyasl
        import matplotlib.pylab as plt
        import numpy as np

        p = pyasl.Pizzolato2003()

        # Define array of rotation periods [days]
        prot = np.arange(0.2, 30, 0.1)

        lx = np.zeros(prot.size)
        lxlbol = np.zeros(prot.size)

        # B-V color of star
        bv = 0.7

        # Obtain ...
        for i in range(prot.size):
            # ... log10 of X-ray luminosity
            lx[i] = p.log10lxbv(bv, prot[i])[0]
            # ... and log10(Lx/Lbol)
            lxlbol[i] = p.log10lxlbolbv(bv, prot[i])[0]

        # Plot result
        plt.subplot(2, 1, 1)
        plt.plot(prot, lx, 'bp-')
        plt.subplot(2, 1, 2)
        plt.plot(prot, lxlbol, 'bp-')
#     plt.show()

    def testsanity_pizzolato(self):
        """
        Pizzolato 2003 sanity
        """
        from PyAstronomy import pyasl
        import matplotlib.pylab as plt
        import numpy as np

        p = pyasl.Pizzolato2003()

        x = p.log10lxbv(0.87, 1.0)
        self.assertAlmostEqual(
            x[0], 29.9, msg="Lx saturation level for b-v=0.87 does not match.", delta=1e-7)
        self.assertAlmostEqual(
            x[1], 0.3, msg="Lx error for saturation level for b-v=0.87 does not match.", delta=1e-7)

        x = p.log10lxlbolmass(1.15, 3.0)
        self.assertAlmostEqual(
            x[0], -4.1694, msg="Pizzolato relation (m=1.15, pr=3.0) failed (value=" + str(x[0]) + ").", delta=1e-4)

    def testsanity_expCorrRN_Example1(self):
        """
        Sanity of example 1 for expCorrRN
        """
        import matplotlib.pylab as plt
        from PyAstronomy import pyasl

        # Generate 200 exponentially correlated Gaussian
        # random numbers with a decay time of 5
        c1 = pyasl.expCorrRN(200, 5)

        # Generate 200 exponentially correlated Gaussian
        # random numbers with decay time 10, mean 4, and
        # standard deviation of 2.3.
        #
        # The results are: The correlated random numbers,
        # the uncorrelated numbers used as input, and the
        # correlated coefficient (exp(-1/tau)).
        c2, g, f = pyasl.expCorrRN(200, 10, mean=4.0, std=2.3, fullOut=True)

        plt.subplot(2, 1, 1)
        plt.plot(range(200), c1, 'bp-')
        plt.subplot(2, 1, 2)
        plt.plot(range(200), c2, 'bp-')
        plt.plot(range(200), g, 'g.')
#     plt.show()

    def testsanity_expCorrRN_Example2(self):
        """
        Sanity of example 2 for expCorrRN
        """
        import numpy as np
        import matplotlib.pylab as plt
        from PyAstronomy import pyasl

        # Generate n exponentially correlated Gaussian
        # random numbers with a decay time, tau
        n = 500
        tau = 5.
        c1 = pyasl.expCorrRN(n, tau)

        # Obtain autocorrelation function
        ac = np.correlate(c1, c1, mode="full")[n - 1:]

        # Plot correlated random numbers and autocorrelation
        # function along with exponential model.
        x = np.arange(float(n))
        plt.subplot(2, 1, 1)
        plt.plot(x, c1, 'bp-')
        plt.subplot(2, 1, 2)
        plt.plot(x, ac, 'b.')
        plt.plot(x, np.exp(-x / tau) * ac.max(), 'r--')
    #   plt.show()

    def testsanity_ramirez2005Example(self):
        """
        Check sanity of Ramirez/Melendez 2005 example
        """
        from PyAstronomy import pyasl

        # Create class instance
        r = pyasl.Ramirez2005()

        # Which color bands are available
        print("Available color bands: ", r.availableBands())

        # Convert B-V to effective temperature and back
        bv = 0.75
        feh = 0.0
        teff = r.colorToTeff("B-V", bv, feh)
        bv1 = r.teffToColor("B-V", teff, feh)
        # Watch out for differences between input bv and the output bv1
        print("B-V = ", bv, ", Teff = ", teff,
              ", bv1 = ", bv1, ", bv-bv1 = ", bv - bv1)

    def testsanity_ramirez2005Example2(self):
        """
        Checking Ramirez, Ballesteros example
        """
        from PyAstronomy import pyasl

        b = pyasl.BallesterosBV_T()
        r = pyasl.Ramirez2005()

        # Convert B-V to effective temperature and back
        for bv in [0.35, 0.45, 0.55, 0.65, 0.75, 0.85, 0.95, 1.05, 1.15, 1.25, 1.35, 1.45]:
            tr = r.colorToTeff("B-V", bv, 0.0)
            tb = b.bv2T(bv)

            print(("B-V [mag] = {3:4.2f} : Teff (R05) = {0:4.0f} K, " +
                   "Teff (B12) = {1:4.0f} K, dTeff = {2: 4.0f} K").format(tr, tb, tr - tb, bv))

    def testsanity_ramirez2005(self):
        """
        Check sanity Ramirez/Melendez 2005 conversions
        """
        from PyAstronomy import pyasl
        import numpy as np
        from itertools import product

        feh = 0.0
        r = pyasl.Ramirez2005()
        # Compare with numbers given in Allen (astrophysical quantities, 4th edition)
        for teff, bv in zip([7000., 5950., 5310., 4410.], [0.35, 0.58, 0.74, 1.15]):
            self.assertAlmostEqual(bv, r.teffToColor("B-V", teff, feh), delta=0.03,
                                   msg="B-V conversion (Teff " + str(teff) + ") out of sync with Allen")

        for teff, bv in zip([6500., 6000., 5000.], [0.286, 0.36, 0.535]):
            self.assertAlmostEqual(bv, r.teffToColor("b-y", teff, feh), delta=0.01,
                                   msg="B-V conversion (Teff " + str(teff) + ") out of sync with Allen")

        for teff, bv in zip([5000., 4500., 4000.], [0.433, 0.51, 0.735]):
            self.assertAlmostEqual(bv, r.teffToColor("R_C-I_C", teff, feh, 'g'), delta=0.04,
                                   msg="R_C-I_C conversion (Teff " + str(teff) + ") out of sync with Allen (giants)")

        for teff, bv in zip([5050., 4660., 4050.], [0.86, 1., 1.5]):
            self.assertAlmostEqual(bv, r.teffToColor("B-V", teff, feh, 'g'), delta=0.1,
                                   msg="B-V conversion (Teff " + str(teff) + ") out of sync with Allen (giants)")

        # Check consistency
        bv = np.arange(0.4, 1.01, 0.05)
        feh = np.arange(-2., 0.3, 0.2)
        for x in product(bv, feh):
            teff = r.colorToTeff("B-V", x[0], x[1], 'ms')
            col = r.teffToColor("B-V", teff, x[1], 'ms')
            self.assertAlmostEqual(x[0], col, delta=1e-2,
                                   msg="Two-way color-temperature conversion is inconsistent. bv = %.4e, feh = %.4e" % x)

    def testsanity_BallesterosExample(self):
        """
        Checking example for Ballesteros
        """
        from PyAstronomy import pyasl

        b = pyasl.BallesterosBV_T()

        bv = 0.65

        # Convert B-V into effective temperature
        teff = b.bv2T(0.65)
        print("B-V = {0:4.2f} mag -> Teff = {1:4.0f} K".format(bv, teff))

        # Convert effective temperature into B-V color
        teff = 4568.0
        bv = b.t2bv(teff)
        print("Teff = {0:4.0f} K -> B-V = {1:4.2f} mag".format(teff, bv))

    def testsanity_gyroAgeBarnes(self):
        """
        Check Gyro age of Barnes 2007
        """
        # Eq 17, 18 (B-V, period, relative error)
        dat = [[0.5, 7, 20], [0.65, 12, 15], [1.0, 20, 13], [1.5, 30, 13]]
        for d in dat:
            a, e = pyasl.gyroAgeBarnes(d[1], d[0])
            self.assertAlmostEqual(e / a * 100., d[2], delta=1.0,
                                   msg="Relative errors of Barnes 2007 cannot be reproduced.")

    def testsanity_sanity_gyroAgeBarnesExample(self):
        """
        Example gyro age (Barnes 2007)
        """
        from PyAstronomy import pyasl

        # Parameters of the Sun (Barnes 2007, p 1174)
        bv = 0.642
        p = 26.09

        # Obtain solar age ...
        age = pyasl.gyroAgeBarnes(p, bv)
        # ... and print it
        print("Solar age: {0:4.2f} +/- {1:4.2f} Ga".format(*age))

    def testsanity_chromoAgeRHK(self):
        """
        Checking sanity of chromospheric age (Donahue)
        """
        import numpy as np
        # R'HK vs log10(age) (estimated from Fig. 1 in Donahue 1998)
        dat = [[-4.25, 7.], [-4.6, 9.1], [-5.0, 9.7]]
        for d in dat:
            a = np.log10(pyasl.chromoAgeRHK(d[0]) * 1e9)
            print(a, d[1])
            self.assertAlmostEqual(a, d[1], delta=0.3,
                                   msg="Cannot reproduce age estimate from Donahue fig 1.")

    def testsanity_chromoAgeRHKExample(self):
        """
        Example chromospheric age (Donahue)
        """
        from PyAstronomy import pyasl

        # Approximate chromospheric age of the Sun
        print("Solar age: {0:4.2f} Ga".format(pyasl.chromoAgeRHK(-4.95)))

    def testsanity_abundancepatternsExample(self):
        """
        Checking example of AbundancePatterns
        """
        from PyAstronomy import pyasl

        ap = pyasl.AbundancePatterns()

        print("Names of the available abundance patterns:")
        print(ap.availablePatterns())

        print()
        print("Get the Asplund et al. pattern (aspl) as a dictionary using")
        print("atomic number as a key:")
        print(ap.pattern("aspl", form="dict", key="number"))

        print()
        print("Get (relative) number abundance of oxygen using elemental symbol:")
        print(ap.abundance("O", pat="wilm"))
        print("or atomic number")
        print(ap.abundance(8, pat="wilm"))
        
        print()
        for el, ab in ap.patternByMass("aspl", key="symbol").items():
            print(f"Element: {el} with mass fraction of {ab}")

    def testsanity_abundancePatterns(self):
        """
        Checking abundance pattern
        """
        from PyAstronomy import pyasl

        ap = pyasl.AbundancePatterns()

        self.assertAlmostEqual(2.82e-07, ap.abundance("P", pat="feld"), delta=1e-12,
                               msg="Abundance of P (feld) incorrect.")
        p = ap.pattern("wilm", form="dict", key="symbol")
        self.assertAlmostEqual(p["Ti"], 6.46e-08, delta=1e-12,
                               msg="Ti abundance broken (wilm)")

    def testsanity_SpecTypeDeJagerExample1(self):
        """
        Sanity of example 1 (SpecTypeDeJager)
        """
        from PyAstronomy import pyasl

        # Instantiate class object
        sdj = pyasl.SpecTypeDeJager()

        llum, lteff = sdj.lumAndTeff("K0", "V")

        print("Luminosity = {0:4.2f} Lsun".format(10.0**llum))
        print("Effective temperature = {0:6.1f} K".format(10.0**lteff))

    def testsanity_SpecTypeDeJagerExample2(self):
        """
        Sanity of example 2 (SpecTypeDeJager)
        """
#     from __future__ import print_function
        from PyAstronomy import pyasl
        import matplotlib.pylab as plt

        # Instantiate class object
        sdj = pyasl.SpecTypeDeJager()

        # Set luminosity class
        lk = "V"

        # Save spectral types, log(teff), and log(luminosity)
        spts = []
        lteffs = []
        llums = []

        # Save information to annotate abscissa
        xt = []
        xtl = []

        for t in "OBAFGKM":
            for n in range(10):
                if (t == "O") and (n == 0):
                    # Skip the invalid "O0" type
                    continue

                # Save the spectral type
                spts.append(t + str(n))

                # Get log10 of luminosity and effective temperature
                ll, lt = sdj.lumAndTeff(spts[-1], lk)
                # and save to lists
                llums.append(ll)
                lteffs.append(lt)

                # Save location (i.e., number in the list) and
                # spectral for annotating the abscissa
                if (n == 0) or (n == 5):
                    xt.append(len(spts) - 1)
                    xtl.append(spts[-1])


#     ax1 = plt.subplot(2,1,1)
#     # Plot log10(effective temperature)
#     plt.plot(lteffs)
#     plt.ylabel("$\log_{10}$(T$_{eff}$)")
#     plt.setp(ax1, xticks=xt,xticklabels=xtl)
#     ax2 = plt.subplot(2,1,2)
#     # Plot log10(luminosity)
#     plt.plot(llums)
#     plt.ylabel("$\log_{10}$($L/L_{\odot}$)")
#     plt.setp(ax2, xticks=xt,xticklabels=xtl)
#     plt.xlabel("Spectral type")
#     plt.show()

    def testsanity_SpecTypeDeJagerSanity(self):
        """
        Sanity of SpecTypeDeJager
        """
        from PyAstronomy import pyasl

        # Instantiate class object
        sdj = pyasl.SpecTypeDeJager()

        llum, lteff = sdj.lumAndTeff("K0", "V")
        self.assertAlmostEqual(10.0**(lteff - 3.712), 1.,
                               delta=0.05, msg="Mismatch K0 V (teff)")
        self.assertAlmostEqual(10.0**(llum + 0.258), 1.,
                               delta=0.2, msg="Mismatch K0 V (lum)")

        llum, lteff = sdj.lumAndTeff("B7", "Ia")
        self.assertAlmostEqual(10.0**(lteff - 4.072), 1.,
                               delta=0.05, msg="Mismatch B7 Ia (teff)")
        self.assertAlmostEqual(10.0**(llum - 5.123), 1.,
                               delta=0.2, msg="Mismatch B7 Ia (lum)")

        llum, lteff = sdj.lumAndTeff("G4", "IV")
        self.assertAlmostEqual(10.0**(lteff - 3.723), 1.,
                               delta=0.05, msg="Mismatch G4 IV (teff)")
        self.assertAlmostEqual(10.0**(llum - 0.802), 1.,
                               delta=0.2, msg="Mismatch G4 IV (lum)")

        llum1, lteff1 = sdj.lumAndTeff("G4", "V")
        llum2, lteff2 = sdj.lumAndTeff("G4.5", "V")
        llum3, lteff3 = sdj.lumAndTeff("G5", "V")
        self.assertTrue(llum1 > llum2 > llum3,
                        msg="Float subtype, wrong order of luminosities")
        self.assertTrue(lteff1 > lteff2 > lteff3,
                        msg="Float subtype, wrong order of luminosities")

    def testsanity_MMSCETSTableExample(self):
        """
        Sanity of example for MMSCETS table
        """
        from PyAstronomy import pyasl
        import matplotlib.pylab as plt

        # Instantiate class
        m = pyasl.MMSCETSTable()

        # Print the entire data file
        for l in m.getContent():
            print(l, end='')

        print()
        print("Available columns: ", ", ".join(m.availableColumns()))

        # Get the tabulated data as an ASCII table
        td = m.getTable()

        # Plot absolute visual brightness vs. effective temperature
        plt.plot(td["Teff"], td["Mv"], 'b.-')
        plt.xlabel("Teff [K]")
        plt.ylabel("Mv [mag]")
#     plt.show()

    def testsanity_Roche(self):
        """
        Check Roche
        """
        from PyAstronomy import pyasl
        import numpy as np
        
        # From Monacki 1984 (Table 1)
        qdat = ((0.01, 3.16664, 3.15345, 0.85853, 1.15622),
                (0.23, 3.78419, 3.54726, 0.64581, 1.45853),
                (0.84, 3.99643, 3.48615, 0.51794, 1.66962),
                (0.54, 3.95665, 3.54129, 0.56301, 1.59529)
            )
        
        for qd in qdat:
            q = qd[0]
            x1, p1 = pyasl.get_lagrange_1(q)
            x2, p2 = pyasl.get_lagrange_2(q)
            self.assertAlmostEqual(qd[1], p1, delta=1e-4, msg="Roche: problem with Monacki " + str(qd))
            self.assertAlmostEqual(qd[2], p2, delta=1e-4, msg="Roche: problem with Monacki " + str(qd))
            self.assertAlmostEqual(qd[3], x1, delta=1e-4, msg="Roche: problem with Monacki " + str(qd))
            self.assertAlmostEqual(qd[4], x2, delta=1e-4, msg="Roche: problem with Monacki " + str(qd))

    def testsanity_roche_example(self):
        """ Cehcking example of Roche """
        from PyAstronomy import pyasl
        import numpy as np
        import matplotlib.pylab as plt
        
        x, y = np.linspace(-1.5,2,300), np.linspace(-1.6,1.6,300)
        xx, yy = np.meshgrid(x, y)
        # Coordinates in orbital plain
        z = 0
        
        # Mass ratio
        q = 0.2
        
        # Get dimensional values of Roche potential
        p = pyasl.rochepot_dl(xx, yy, z, q)
        
        # Positions (and potentials) of Lagrange points
        l1, l1pot = pyasl.get_lagrange_1(q)
        l2, l2pot = pyasl.get_lagrange_2(q)
        l3, l3pot = pyasl.get_lagrange_3(q)
        l4, l5 = pyasl.get_lagrange_4(), pyasl.get_lagrange_5()
        l4pot = pyasl.rochepot_dl(l4[0], l4[1], l4[2], q)
        l5pot = pyasl.rochepot_dl(l5[0], l5[1], l5[2], q)
        
        print("Effective (dimensionless) radii of first and second mass")
        print("According to the approximation of Eggleton 1983:")
        r1eff = pyasl.roche_lobe_radius_eggleton(q, 1)
        r2eff = pyasl.roche_lobe_radius_eggleton(q, 2)
        print("    Reff1: %5.3f" % r1eff)
        print("    Reff2: %5.3f" % r2eff)
        print()
        print("Roche volume and effective radius from Monte Carlo integration:")
        mcvol1 = pyasl.roche_vol_MC(q,1)
        mcvol2 = pyasl.roche_vol_MC(q,2)
        print("    MC Roche lobe volume 1: %6.4f +/- %6.4f" % (mcvol1[0:2]))
        print("    MC Roche lobe volume 2: %6.4f +/- %6.4f" % (mcvol2[0:2]))      
        print("    MC effective radius 1: %6.4f +/- %6.4f" % (mcvol1[2:]))
        print("    MC effective radius 2: %6.4f +/- %6.4f" % (mcvol2[2:]))   
        
#         plt.contour(p, [l5pot*1.02, l3pot, l2pot, l1pot], colors=['g', 'c', 'b', 'r'], extent=[-1.5,2,-1.6,1.6])

    def testsanity_roche(self):
        """ Sanity checks of Roche and root finding """
        from PyAstronomy import pyasl
        from PyAstronomy import constants as PC
        
        def yp(x):
            return (x-0.1)*(x-0.5)*(x-0.7)*(x-0.5000002)
         
        rro = [0.1, 0.5, 0.7]
        roots = sorted(pyasl.bisect_root_find(yp, 0, 1, 4, 1e-4, bqargs={}))
        self.assertEqual(len(rro), len(roots), msg=f"Problem with the number of roots in bisect root finding. Roots are {roots}")
        for i, r in enumerate(roots):
            self.assertAlmostEqual(r, rro[i], 5, msg=f"Problem with bisect root finding. Roots are {roots}")
            
        pc = PC.PyAConstants()
        pc.setSystem("SI")
        q = pc.MEarth/pc.MSun
        rads = pyasl.get_epradius_ss_polar_side(q, pot=None)
        rads = [r*pc.AU/1e3 for r in rads]
        print(rads)
        self.assertAlmostEqual(rads[0], 1.5e6, delta=1e4, msg=f"Problem with L1 height of Earth. Radii are {rads}")

    def testsanity_roche_example2(self):
        """
        Roche lobe example 2 (radii of Earth and hot Jupiter)
        """
        from PyAstronomy import pyasl
        from PyAstronomy import constants as PC
        
        # Shape of the Earth and its Roche lobe (no rotation of Earth)
        pc = PC.PyAConstants()
        pc.setSystem("SI")
        
        # Mass ratio (m2/m1)
        q = pc.MEarth/pc.MSun
        print(f"Earth/Sun mass ratio = {q}")
        
        # Radii of Roche lobe along Earth--Sun connecting line, polar (out-of-plane),
        # and side (in plane)
        rads = pyasl.get_epradius_ss_polar_side(q, pot=None)
        # Convert into km
        rads = [r*pc.AU/1e3 for r in rads]
        print(f"Roche lobe radii (substellar, polar, side) [1e6 km] = " + \
            ", ".join(["%g"%(r/1e6) for r in rads]))
        
        reff_earth = pc.REarth/pc.RJ
        # Small epsilon because rocky Earth is a small body on a large
        # orbit (by Roche standards)
        erads = pyasl.get_radius_ss_polar_side(q, 1, reff_earth, eps=1e-10)
        
        # Convert into km
        erads = [r*pc.RJ/1e3 for r in erads]
        print(f"Earth radii (substellar, polar, side) [km] = " + \
            ", ".join(["%g"%(r) for r in erads[0:3]]))
        
        print()
        print("Roche lobe and planetary shape of typical hot Jupiter")
        # Typical hot Jupiter
        q = 1e-3
        # sma in au
        sma = 0.02
        # Effective (circular disk) transit radius [RJ]
        reff = 1.4
        
        rads = pyasl.get_epradius_ss_polar_side(q, pot=None)
        # Convert into km
        rads = [r*pc.AU/1e3 for r in rads]
        print(f"Roche lobe radii (substellar, polar, side) [1e6 km] = " + \
            ", ".join(["%g"%(r/1e6) for r in rads]))
        
        jrads = pyasl.get_radius_ss_polar_side(q, sma, reff, eps=1e-10)
        print(f"Hot Jupiter radii (substellar, polar, side) [RJ] = " + \
            ", ".join(["%g"%(r) for r in jrads[0:3]]))

    def testsanity_roche_limit_fluid(self):
        """
        Test sanity of roche_limit_fluid
        """
        from PyAstronomy import constants as PC
        from PyAstronomy import pyasl
        pc = PC.PyAConstants()
        pc.setSystem("SI")
        a_r_jup = pyasl.roche_limit_fluid(pc.MJ/pc.MSun, pc.RJ)
        self.assertAlmostEqual(a_r_jup/1e3, 1_713_025, delta=60_000, msg=f"Roche lobe limit for Jupiter does not fit ({a_r_jup} km).")

    def testsanity_sysrem_example(self):
        """
        Checking SysRem example
        """
        from PyAstronomy import pyasl
        import numpy as np
        import matplotlib.pylab as plt
        
        # No. of data sets (e.g., light curves)
        nds = 50
        # No. of data points per data set
        nobs = 200
        
        obs, sigs = [], []
        
        # Generate mock data
        x = np.linspace(0,1,nobs)-0.5
        for i in range(nds):
            # Add noise
            y = np.random.normal(0,0.01+0.0001*i,nobs)
            # Add 3rd degree polynomial
            p = (i, 0.2*i, 0.3*i)
            y += np.polyval(p,x)
            # Add moving Gaussian signal
            y -= 0.02 * np.exp(-(x-(i/nds-0.5))**2/(2*0.02**2))
            # Add growing but stationary Gaussian signal
            y -= (i**2*0.05) * np.exp(-x**2/(2*0.06**2))
            obs.append(y)
            sigs.append(np.ones_like(y)*0.01+0.0001*i)
            
            #plt.plot(x, y, '.-')
        #plt.title("Mock data")
        #plt.show()
        
        sr = pyasl.SysRem(obs, sigs)
        # First iteration
        r, a, c = sr.iterate()
        #plt.subplot(2,1,1)
        #plt.title("Residuals after first (top) and second iteration (bottom)")
        #plt.imshow(r, origin='lower', aspect="auto")
        # Second iteration
        r, a, c = sr.iterate()
        #plt.subplot(2,1,2)
        #plt.imshow(r, origin='lower', aspect="auto")
        #plt.show()

    def testsanity_sysrem_lindata(self):
        """
        Checking SysRem sanity for linear data
        """
        from PyAstronomy import pyasl
        import numpy as np
        import matplotlib.pylab as plt
        
        np.random.seed(123)
        
        # No. of data sets (e.g., light curves)
        nds = 50
        # No. of data points per data set
        nobs = 200
        
        obs, sigs = [], []
        
        a = np.linspace(0,1,200)
        c = np.random.random(50)
        
        m = np.outer(a,c)
        
        # Generate mock data
        for i in range(nds):
            obs.append(m[::,i])
            sigs.append(np.ones_like(a)*0.01+0.0001*i)
            
        
        sr = pyasl.SysRem(obs, sigs)
        # First iteration
        print(np.std(m))
        r, a, c = sr.iterate()
        print(np.std(r))
        self.assertAlmostEqual(np.std(r), 0, msg="SysRem problem with linear data", delta=1e-10)

    def testsanity_parallacticAngle_example(self):
        """ Checking example of parallactic angle """
        import numpy as np
        import matplotlib.pylab as plt
        from PyAstronomy import pyasl
        
        # Geo-latitude of Keck [deg]
        gl = 19.83
        
        # Declinations to be considered
        decs = np.arange(-80, 81, 20)
        # Hour angles to be considered
        t = np.linspace(0.0, 8*15, 1000)
        
        for dec in decs:
            q = pyasl.parallacticAngle(t, gl, dec)
            plt.plot(t/15, q, '-', label=r"$\delta = $" + f"{dec} deg")
        
        plt.ylabel("Parallactic angle [deg]")
        plt.xlabel("Hour angle [h]")
        plt.legend()
        #plt.show()
        
    def testsanity_sysrem_pca(self):
        """ Check sanity of SYSREM with PCA """
        from PyAstronomy import pyasl
        import numpy as np
        from sklearn.decomposition import PCA
        # n observations (e.g., light curves) with m data points each
        n = 4
        m = 7
        
        # Some arbitrary observations (with observations is COLUMNS)
        obs = np.zeros( (m,n) )
        for i in range(0, n):
            for j in range(m):
                obs[j,i] = j*i**3+j*i**2+(j+i+1)
        # Equal error for all data points
        sigs = np.ones_like(obs)

        pca = PCA()
        res = pca.fit(obs.T)
        
        p1 = pca.components_[0,::]
        p2 = pca.components_[1,::]
        
        sr = pyasl.SysRem(obs, sigs, ms_obs=False, ms_feat=True)
        r1, a1, c1 = sr.iterate()
        r2, a2, c2 = sr.iterate()
        a1 = a1/np.linalg.norm(a1)
        a2 = a2/np.linalg.norm(a2)
        
        sign = lambda x: -1 if x < 0 else 1

        # Get sign right
        if sign(a1[0]) != sign(p1[0]):
            a1 *= -1
        if sign(a2[0]) != sign(p2[0]):
            a2 *= -1    
        
        np.testing.assert_almost_equal(a1, p1, decimal=5, err_msg=f'SYSREM: Problem with PCA component 1. Compared {p1} and {a1} (p1 and a1)', verbose=True)
        np.testing.assert_almost_equal(a2, p2, decimal=5, err_msg=f'SYSREM: Problem with PCA component 2. Compared {p2} and {a2} (p2 and a2)', verbose=True)
                
    def testsanity_sysrem_example(self):
        """
        Checking sanity of SYSREM example
        """
        from PyAstronomy import pyasl
        import numpy as np
        import matplotlib.pylab as plt
        
        # No. of data sets (e.g., light curves)
        nds = 50
        # No. of data points per data set
        nobs = 200
        
        obs, sigs = [], []
        
        # Generate mock data
        x = np.linspace(0,1,nobs)-0.5
        for i in range(nds):
            # Add noise
            y = np.random.normal(0,0.01+0.0001*i,nobs)
            # Add 3rd degree polynomial
            p = (i, 0.2*i, 0.3*i)
            y += np.polyval(p,x)
            # Add moving Gaussian signal
            y -= 0.02 * np.exp(-(x-(i/nds-0.5))**2/(2*0.02**2))
            # Add growing but stationary Gaussian signal
            y -= (i**2*0.05) * np.exp(-x**2/(2*0.06**2))
            obs.append(y)
            sigs.append(np.ones_like(y)*0.01+0.0001*i)
        
            plt.plot(x, y, '.-')
        plt.title("Mock data")
        # plt.show()
        
        sr = pyasl.SysRem(obs, sigs)
        # First iteration
        r, a, c = sr.iterate()
        # plt.subplot(2,1,1)
        # plt.title("Residuals after first (top) and second iteration (bottom)")
        # plt.imshow(r, origin='lower', aspect="auto")
        # Second iteration
        r, a, c = sr.iterate()
        # plt.subplot(2,1,2)
        # plt.imshow(r, origin='lower', aspect="auto")
        # plt.show()

    def testsanity_scaleHeight(self):
        """
        Checking sanity of scale height
        """
        from PyAstronomy import pyasl
        T, mu, g = 290, 28.97, 9.8
        she = pyasl.atmosphericScaleHeight(T, mu, g)
        she2 = pyasl.atmosphericScaleHeight_MR(290, 28.97, 1, 1, "E")
        self.assertAlmostEqual(she/she2, 1, delta=1e-3, msg="Scale height of Earth cross check failed")
        
        T, mu, mp, rp = 165, 2.2, 1, 1
        shj = pyasl.atmosphericScaleHeight_MR(T, mu, mp, rp, "J")
        self.assertAlmostEqual(shj, 25.152, delta=1e-3, msg=f"Scale height of Jupiter does not match. result obtained is {shj}")
    
    def testsanity_scaleHeight_example(self):
        """
        Checking example of scale height
        """
        from PyAstronomy import pyasl
    
        T, mu, g = 290, 28.97, 9.8
        she = pyasl.atmosphericScaleHeight(T, mu, g) 
        
        print("Earth")
        print(f"T, mu, g = {T} K, {mu}, {g} m/s**2")
        print(f"Scale height = {she:4.1f} [km]")
        
        
        T, mu, mp, rp = 165, 2.2, 1, 1
        shj = pyasl.atmosphericScaleHeight_MR(T, mu, mp, rp, "J")
        
        print("Jupiter")
        print(f"T, mu, mp, rp = {T} K, {mu}, {mp} [MJ], {rp} [RJ]")
        print(f"Scale height = {shj:4.1f} [km]")

    def testsanity_solarDirectFluxMeinelExample(self):
        """
        Checking example of solarDirectFluxMeinel
        """
        from PyAstronomy import pyasl
        import numpy as np
        # import matplotlib.pylab as plt
        
        z = np.linspace(0,89.9,200)
        
        for h in [0,2,4]:
        
            f = pyasl.solarDirectFluxMeinel(z, height=h)
        
            # plt.plot(z, f, '-', label=f"Height = {h} km")
            
        # plt.legend()
        # plt.xlabel("Zenith angle [deg]")
        # plt.ylabel("Direct solar flux [W/m**2]")
        # plt.show()


# -*- coding: utf-8 -*-
from __future__ import print_function, division
import numpy
import os
from . import astroTimeLegacy as at
from . import aitoffLegacy
import unittest
from .baryvel import baryvel, helcorr
from . import eq2hor
from . import sunpos
from . import moonpos
from . import moonphase
from . import posAngle
from .airtovac import airtovac2, vactoair2
import six.moves as smo

#  How to run...
#  p3 -m PyAstronomy.pyasl.asl.test
#

class AstroTimeLegacyTest(unittest.TestCase):

  def setUp(self, tddir="testPro", p=1e-6):
    """
      Parameter:
        tddir - Directory where test data can be found.
    """
    self.tddir = tddir
    self.p = p

  def getData(self, fn):
    if not os.path.isfile(self.tddir+"/"+fn):
      print("Could not find test data file: ", fn)
      print("  Use test.pro (create_test_data) to generate test data.")
      return [False, None]
    return [True, numpy.loadtxt(self.tddir+"/"+fn)]

  def test_bprecess(self):
    fn = "bprecess.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    
    err = 0
    for i in smo.range(len(dat[::,0])):
      ra, de, r5, d5, pa1, pa, m1_1, m2_1, m1_2, m2_2, rv1, rv = dat[i,::]
      mu = [m1_1, m2_1]
      if m1_1 == 0.0 and m2_1 == 0.0:
        mu = None
      if pa1 == 0.0:
        pa = 0.0
      if rv1 == 0.0:
        rv = 0.0
      
      r = at.bprecess(ra, de, parallax=pa, mu_radec=mu, rad_vel=rv)
      # [ra_1950, dec_1950, MU_RADEC, PARALLAX, RAD_VEL]
      
      if (r5-r[0])/r5 > self.p:
        print("Problem r5: ", (r5-r[0])/r5)
        err += 1
        print(dat[i,::])
        print(r)
        print("")
        continue
      if (d5-r[1])/d5 > self.p:
        print("Problem d5: ", (d5-r[1])/d5)
        err += 1
        print(dat[i,::])
        print(r)
        print("")
        continue
      if r[2] is not None:
        if (r[2][0]-m1_2)/m1_2 > self.p:
          print("Problem m1: ", (r[2][0]-m1_2)/m1_2)
          err += 1
          print(dat[i,::])
          print(r)
          print("")
          continue
        if (r[2][1]-m2_2)/m2_2 > self.p:
          print("Problem m2: ", (r[2][1]-m2_2)/m2_2)
          err += 1
          print(dat[i,::])
          print(r)
          print("")
          continue
      if r[3] != 0.0:
        if (r[3]-pa)/pa > self.p:
          print("Problem parallax: ", (r[3]-pa)/pa)
          err += 1
          print(dat[i,::])
          print(r)
          print("")
          continue
      if rv != 0.0:
        if (r[4]-rv)/rv > self.p:
          print("Problem rv: ", (r[4]-rv)/rv)
          err += 1
          print(dat[i,::])
          print(r)
          print("")
          continue
    
    self.assertEqual(err, 0)

  def test_premat(self):
    fn = "premat.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    err = 0
    for i in smo.range(len(dat[::,0])):
      r = numpy.reshape(dat[i,2:11], (3,3))
      if dat[i,11] == 0:
        m = at.premat(dat[i,0], dat[i,1])
      else:
        m = at.premat(dat[i,0], dat[i,1], FK4=True)
      if (r-m).max() > self.p:
        print("test_premat - Error detected")
        err += 1
    self.assertEqual(err, 0)

  def test_precess(self):
    fn = "precess.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    err = 0
    for i in smo.range(len(dat[::,0])):
      fk4 = True
      rad = True
      if dat[i,6] == 0.0:
        fk4 = False
      if dat[i,7] == 0.0:
        rad = False
      r = at.precess(dat[i,2], dat[i,3], dat[i,0], dat[i,1], FK4=fk4, radian=rad)
      if (r[0]-dat[i,4])/r[0] > self.p:
        err += 1
      if (r[1]-dat[i,5])/r[0] > self.p:
        err += 1
    self.assertEqual(err, 0)


  def test_precess_xyz(self):
    fn = "precess_xyz.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    err = 0
    for i in smo.range(len(dat[::,0])):
      x,y,z = at.precess_xyz(dat[i,2], dat[i,3], dat[i,4], dat[i,0], dat[i,1])
      if abs((x-dat[i,5])/x) > self.p:
        err += 1
        print("Probelm x: ", (x-dat[i,5])/x)
        print(x,y,z)
        print(dat[i,5:8])
        print(numpy.array([x,y,z]) - dat[i,5:8])
        print("")
        continue
      if abs((y-dat[i,6])/y) > self.p:
        err += 1
        print("Probelm y: ", (y-dat[i,6])/y)
        print(x,y,z)
        print(dat[i,5:8])
        print(numpy.array([x,y,z]) - dat[i,5:8])
        print("")
        continue
      if abs((z-dat[i,7])/z) > self.p:
        err += 1
        print("Probelm z: ", (z-dat[i,7])/z)
        print(x,y,z)
        print(dat[i,5:8])
        print(numpy.array([x,y,z]) - dat[i,5:8])
        print("")
        continue
    self.assertEqual(err, 0)

  def test_xyz(self):
    fn = "xyz.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    err = 0
    for i in smo.range(len(dat[::,0])):
      r = numpy.array(at.xyz(dat[i,0], velocity=True, equinox=dat[i,7]))
      m = (numpy.abs(r-dat[i,1:7])/r).max()
      if m > self.p:
        err += 1
        print("Problem: ", m)
        print(r)
        print(m)
        print("")
        continue
    self.assertEqual(err, 0)

  def test_helio_jd(self):
    fn = "helio_jd.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    err = 0
    for i in smo.range(len(dat[::,0])):
      hjd = at.helio_jd(dat[i,1], dat[i,2], dat[i,3])
      if abs((hjd-dat[i,0])/hjd) > self.p:
        print("Problem HJD: ", abs((hjd-dat[i,0])/hjd))
        print(hjd, dat[i,0])
        print("")
        err += 1
        continue
    self.assertEqual(err, 0)

  def test_daycnv(self):
    fn = "daycnv.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    err = 0
    for i in smo.range(len(dat[::,0])):
      r = numpy.array(at.daycnv(dat[i,0]))
      m = numpy.abs((r - dat[i,1::])/r).max()
      if m > self.p:
        print("Problem: ")
        print(r)
        print(dat[i,1::])
        err += 1
        continue
    self.assertEqual(err, 0)
  
  def test_aitoff(self):
    fn = "aitoff.test"
    good, dat = self.getData(fn)
    self.assertEqual(good, True)
    err = 0
    for i in smo.range(len(dat[::,0])):
      x, y = aitoffLegacy.aitoff(dat[i,0], dat[i,1])
      # The second condition (abs(x)>1e-2)) because IDL uses single precision
      if (abs((x-dat[i,2])/x) > 1e-4 and (abs(x)>1e-2)) or (abs((y-dat[i,3])/y) > 1e-4 and (abs(y) > 1e-2)):
        print("Problem: ")
        print("  l, b, x(IDL), y(IDL), x(Python), y(Python): ", dat[i,0], dat[i,1], x, y, dat[i,2], dat[i,3])
        err += 1
    self.assertEqual(err, 0)

  def test_inverseAitoff(self):
    err = 0
    for i in smo.range(1000):
      l = numpy.random.random() * 360.0 - 180.0
      b = numpy.random.random() * 180.0 - 90.0
      x, y = aitoffLegacy.aitoff(l, b)
      l2, b2 = aitoffLegacy.inverseAitoff(x, y)
      if abs(l-l2) > 1e-6 or abs(b-b2) > 1e-6:
        print("Problem: ")
        print("  ", l, b, x, y, l2, b2)
        err += 1
    self.assertEqual(err, 0)
  
  def test_baryvel(self):
    dat = numpy.loadtxt("testPro/baryvel.test")
    err = 0
    for i in range(10000):
      jd = 2.4e6 + float(i)*10.37681
      a, b =  baryvel(jd, 0)
      delta = numpy.abs(dat[i,::] - numpy.concatenate((a,b)))
      if max(delta) > 1e-6:
        err += 1
        print("Baryvel error:", dat[i,::], a, b)
        print("max: ", max(delta))
    self.assertEqual(err, 0)

    i = 0
    dat = numpy.loadtxt("testPro/baryvel2.test")
    for j in range(101):
      jd = 2.4e6 + float(j)*1000.37681
      for k in range(11):
        deq = jd + float(k-5) * 10000.0
        a, b =  baryvel(jd, deq)
        delta = numpy.abs(dat[i,0:6] - numpy.concatenate((a,b)))
        
        if abs(jd - dat[i,6]) > 1e-6:
          print("JD problem: ", jd, dat[i,6], i, k)
        if abs(deq - dat[i,7]) > 1e-6:
          print("deq problem: ", deq, dat[i,7], i, k)
        
        if max(delta) > 1e-6:
          err += 1
          print("Baryvel error:", dat[i,::], a, b)
          print("max: ", max(delta))
          print("i, k: ", i, k)
        i += 1


class IDLTests(unittest.TestCase):

  def setUp(self, tddir="testPro", p=1e-6):
    """
      Parameter:
        tddir - Directory where test data can be found.
    """
    self.tddir = tddir
    self.p = p

  def getData(self, fn):
    if not os.path.isfile(self.tddir+"/"+fn):
      print("Could not find test data file: ", fn)
      print("  Use test.pro (create_test_data) to generate test data.")
      return [False, None]
    return [True, numpy.loadtxt(self.tddir+"/"+fn)]
    
  def test_sunpos(self):
    """
      Testing sunpos
    """
    dat = self.getData("sunpos.test")[1]
    for i in smo.range(len(dat[::,0])):
      jd, ra, dec, longmed, oblt = sunpos.sunpos(dat[i,0], full_output=True)
      self.assertAlmostEqual(ra, dat[i,1], delta=self.p)
      self.assertAlmostEqual(dec, dat[i,2], delta=self.p)
      self.assertAlmostEqual(longmed, dat[i,3], delta=self.p)
      self.assertAlmostEqual(oblt, dat[i,4], delta=self.p)

  def test_nutate(self):
    dat = self.getData("nutate.test")[1]
    for i in smo.range(len(dat[::,0])):
      l, o = eq2hor.nutate(dat[i,0])
      self.assertAlmostEqual(l*3600.0, dat[i,1], delta=self.p)
      self.assertAlmostEqual(o*3600.0, dat[i,2], delta=self.p)   
    
  def test_co_nutate(self):
    dat = self.getData("co_nutate.test")[1]
    for i in smo.range(len(dat[::,0])):
      dra, ddec, o, dl, do = eq2hor.co_nutate(dat[i,0], dat[i,1], dat[i,2], full_output=True)
      self.assertAlmostEqual(dra*3600./dat[i,3], 1.0, delta=1e-5)
      self.assertAlmostEqual(ddec*3600./dat[i,4], 1.0, delta=1e-5)
      self.assertAlmostEqual(o/180.*numpy.pi/dat[i,5], 1.0, delta=self.p)
      self.assertAlmostEqual(dl*3600./dat[i,6], 1.0, delta=self.p)
      self.assertAlmostEqual(do*3600./dat[i,7], 1.0, delta=self.p)

  def test_co_aberration(self):
    dat = self.getData("co_aberration.test")[1]
    for i in smo.range(len(dat[::,0])):
      dra, ddec = eq2hor.co_aberration(dat[i,0], dat[i,1], dat[i,2])
      self.assertAlmostEqual(dra*3600./dat[i,3], 1.0, delta=self.p)
      self.assertAlmostEqual(ddec*3600./dat[i,4], 1.0, delta=self.p)

  def test_co_refract_forward(self):
    dat = self.getData("co_refract_forward.test")[1]
    for i in smo.range(len(dat[::,0])):
      r = eq2hor.co_refract_forward(dat[i,0], pressure=dat[i,1], temperature=dat[i,2]-273.15)
      self.assertAlmostEqual(r,dat[i,3], delta=self.p)

  def test_co_refract(self):
    dat = self.getData("co_refract.test")[1]
    for i in smo.range(len(dat[::,0])):
      aout, p, t = eq2hor.co_refract(dat[i,0], observer_alt=dat[i,1], pressure=dat[i,2], \
                               temperature=dat[i,3]-273.15, convert_to_observed=True)
      self.assertAlmostEqual(aout/dat[i,4], 1.0, delta=self.p)

  def test_hadec2altaz(self):
    dat = self.getData("hadec2altaz.test")[1]
    for i in smo.range(len(dat[::,0])):
      alt, az = eq2hor.hadec2altaz(dat[i,0], dat[i,1], dat[i,2])
      self.assertAlmostEqual(alt/dat[i,3], 1.0, delta=self.p)
      self.assertAlmostEqual(az/dat[i,4], 1.0, delta=self.p)

  def test_eq2hor(self):
    """
      Testing eq2hor
    """
    dat = self.getData("eq2hor.test")[1]
    for i in smo.range(len(dat[::,0])):
      alt, az, ha = eq2hor.eq2hor(dat[i,2], dat[i,0], dat[i,1], lon=dat[i,3], lat=dat[i,4], alt=dat[i,5])
      self.assertAlmostEqual(alt/dat[i,6], 1.0, delta=self.p*100., msg="Altitude does not fit " + str(dat[i,::]))
      self.assertAlmostEqual(az/dat[i,7], 1.0, delta=self.p*100., msg="Azimuth does not fit " + str(dat[i,::]))
  
  def test_moonpos(self):
    """
      Testing moonpos routine
    """
    dat = self.getData("moonpos.test")[1]
    for i in smo.range(len(dat[::,0])):
      ra, dec, dist, glon, glat = moonpos.moonpos(dat[i,0], False)
      self.assertAlmostEqual(ra/dat[i,1], 1.0, delta=self.p)
      self.assertAlmostEqual(dec/dat[i,2], 1.0, delta=self.p)
      self.assertAlmostEqual(dist/dat[i,3], 1.0, delta=self.p)
      self.assertAlmostEqual(glon/dat[i,4], 1.0, delta=self.p)
      self.assertAlmostEqual(glat/dat[i,5], 1.0, delta=self.p)

  def test_mphase(self):
    """
      Testing moonphase (mphase) routine
    """
    dat = self.getData("mphase.test")[1]
    for i in smo.range(len(dat[::,0])):
      f = moonphase.moonphase(dat[i,0])
      self.assertAlmostEqual(f, dat[i,1], delta=1e-5, msg="Lunar phase does not match." )
      self.assertAlmostEqual(f/dat[i,1], 1.0, delta=1e-5, msg="Lunar phase (relative) does not match")

  def test_posangle(self):
    """
      Testing positionAngle (posAng) routine
    """
    dat = self.getData("posangle.test")[1]
    for i in smo.range(len(dat[::,0])):
      # Using *15.0 because decimal hours have been specified for IDL...
      f = posAngle.positionAngle(dat[i,0]*15.0, dat[i,1], dat[i,2]*15.0, dat[i,3], positive=False)
      self.assertAlmostEqual(dat[i,4], f, delta=1e-9, msg="Position angle does not match.")

  def test_helcorr(self):
    """
      Testing helcorr function for barycentric correction
    """
    dat = self.getData("helcorr.test")[1]
#    import matplotlib.pylab as plt
#    diffs = []
    for i in smo.range(len(dat[::,0])):
      # Using *15.0 because decimal hours have been specified for IDL...
      # The minus accounts for the east-west change
      corr, hjd = helcorr(-dat[i,0], dat[i,1], dat[i,2], dat[i,3]*15.0, dat[i,4], dat[i,5])
#      print corr, dat[i,6], corr-dat[i,6]
#      diffs.append(corr-dat[i,6])
      self.assertAlmostEqual(dat[i,6], corr, delta=1e-5, msg="Barycentric correction does not match.")
      self.assertAlmostEqual(dat[i,7]+2.4e6, hjd, delta=1e-5, msg="HJD does not match.")
#    plt.hist(diffs)
#    plt.show()

  def test_airtovac(self):
    """
      Testing air to vac conversion
    """
    dat = self.getData("airvac2.test")[1]
    wvl = dat[::,0]
    atv = dat[::,1]
    vta = dat[::,2]
    atv2 = airtovac2(wvl)
    vta2 = vactoair2(wvl)
    print("test_airtovac")
    print("  max diff in airtovac", numpy.max(numpy.abs(atv-atv2)))
    print("  max diff in vactoair", numpy.max(numpy.abs(vta-vta2)))
    self.assertAlmostEqual(numpy.max(numpy.abs(atv-atv2)), 0.0, delta=1e-6, msg="Air to Vac conversion inconsistent.")
    self.assertAlmostEqual(numpy.max(numpy.abs(vta-vta2)), 0.0, delta=1e-6, msg="Vac to air conversion inconsistent.")

if __name__ == "__main__":
  suite = unittest.TestLoader().loadTestsFromTestCase(AstroTimeLegacyTest)
  unittest.TextTestRunner(verbosity=2).run(suite)
  
  suite2 = unittest.TestLoader().loadTestsFromTestCase(IDLTests)
  unittest.TextTestRunner(verbosity=2).run(suite2)
